/***************************************************************************
                          fourier.cpp  -  description
                             -------------------
    begin                : mié dic 10 2003
    copyright            : (C) 2003 by projecte
    email                : projecte@localhost.localdomain
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "fourier.h"
#include <iostream.h>



Fourier::Fourier(Imatge &imatge, int mostres){
   int i, nn;

   //inicialitzem les variables dinamiques
   punter_a_Z=NULL;
   v_mostres=NULL;
   nfd=NULL;
   totContorn=NULL;
   
   descriptZ.plensFD=-1;      //inicialitzem els descriptors
   descriptZ.plenspreFD=-2;

   this->mostres=mostres;
   CMasses(imatge);
   
   totContorn=imatge.dw->llista; //llista amb tots els punts del contorn

}


Fourier::~Fourier(){
   if (punter_a_Z!=NULL)
      delete punter_a_Z;

   if (v_mostres!=NULL)
      delete v_mostres;

   if (nfd!=NULL)
      delete nfd;

   if (totContorn!=NULL){
      while (totContorn->first()!=NULL)
         totContorn->removeFirst();
      delete totContorn;
   }
}

void Fourier::CMasses(Imatge &imatge){

  int  Ti=0, Tj=0, Tpunts=0;

  short color_fons=imatge.imatge.data(0,0);
  
  for(int i=0;i<imatge.imatge.dimX;i++){
    for(int j=0;j<imatge.imatge.dimY;j++){
       if(imatge.imatge.data(i,j)!=color_fons){
         Ti+=i;
         Tj+=j;
         Tpunts++;
       }
    }
  }
  imatge.dw->centreX=(float)Ti/Tpunts;
  imatge.dw->centreY=(float)Tj/Tpunts;

}

void Fourier::Fourier1 (Imatge &imatge){        //complex coordinates
   coordenades el;
   int i, total_punts = imatge.dw->llista->count()-1;

   //Fem un canvi de signe de la Y per treballar al 4t quadrant
   el=imatge.dw->llista->first();
   for (i=0;i<total_punts;i++){
      el.Y*=(-1);
      el=imatge.dw->llista->next();
   }
            
   punter_a_Z = new complexe[total_punts];

   el=imatge.dw->llista->first();
   for (int i=0; i<total_punts;i++){
      punter_a_Z[i].x=(double)el.X - imatge.dw->centreX;
      punter_a_Z[i].y=(double)el.Y + imatge.dw->centreY;    //fem la suma perque es el mateix k fer el canvi de signe al centre de masses
      el=imatge.dw->llista->next();
      }

   descriptZ.metode=1;
   descriptZ.plenspreFD=total_punts;

   DescriptorsFourier ();

   escala();
   DescriptorUnic ();
}


void Fourier::Fourier1Equi (Imatge &imatge, bool fft){        //complex coordinates
   int i;                                  
   EquidistaPixels (imatge, mostres); 

   //Fem un canvi de signe a la coordenada Y
   for (i=0;i<mostres;i++)    
      v_mostres[i].y*=(-1.0);
      
   punter_a_Z = new complexe[mostres];

   
   for (int i=0; i<mostres;i++){
      punter_a_Z[i].x=(double)v_mostres[i].x - imatge.dw->centreX;
      punter_a_Z[i].y=(double)v_mostres[i].y + imatge.dw->centreY;      //per tal d treballar al quart quadrant
      }
    
   descriptZ.metode=3;
   descriptZ.plenspreFD=mostres;

   if (fft){
      int r, elevat=0;
      r=1;
      while (r<mostres){
         r*=2;
         elevat++;
      }
      FFT (1, elevat);

   }
   else
      DescriptorsFourier ();
//  DFT();
  
//         AntiTransformada();
   escala();
   DescriptorUnic ();
}


void Fourier::Fourier2 (Imatge &imatge){     //centroid distance
   coordenades el;

   int i, total_punts = imatge.dw->llista->count()-1;

   //Fem un canvi de signe de la Y per treballar al 4t quadrant
   el=imatge.dw->llista->first();
   for (i=0;i<total_punts;i++){
      el.Y*=(-1);
      el=imatge.dw->llista->next();
   }
   
   punter_a_Z = new complexe[total_punts];

       
   el=imatge.dw->llista->first();
   for (int i=0; i<total_punts;i++){
      punter_a_Z[i].x=sqrt ( pow ((el.X - imatge.dw->centreX),2.0) + pow((el.Y + imatge.dw->centreY),2.0));    //sumo CM per fer canvi d quadrant correctament
      punter_a_Z[i].y=0;
      el=imatge.dw->llista->next();
   }

   descriptZ.metode=2;
   descriptZ.plenspreFD=total_punts;
   DescriptorsFourier ();
   escala();
   DescriptorUnic ();
}

void Fourier::Fourier2Equi (Imatge &imatge, bool fft){     //centroid distance

   int i, total_punts = imatge.dw->llista->count()-1;
                                     
   EquidistaPixels (imatge, mostres); //fem 2*total pixels de talls

   //Fem un canvi de signe a la coordenada Y
   for (i=0;i<mostres;i++)
      v_mostres[i].y*=(-1.0);
      
   punter_a_Z = new complexe[mostres];


   for (int i=0; i<mostres;i++){
      punter_a_Z[i].x=sqrt ( pow ((v_mostres[i].x - imatge.dw->centreX),2.0) + pow((v_mostres[i].y + imatge.dw->centreY),2.0));     //sumem per tal de treballar al 4t quadrant
      punter_a_Z[i].y=0.0;
      }

   descriptZ.metode=4;
   descriptZ.plenspreFD=mostres;

   if (fft){
      int r, elevat=0;
      r=1;
      while (r<mostres){
         r*=2;
         elevat++;
      }
      FFT (1, elevat);
   }
   else
      DescriptorsFourier ();

   escala();
   DescriptorUnic ();
}



//Prepara la sortida de dades per pantalla
QString Fourier::MostraPunts(Imatge &imatge){
   QString vector, a;
   coordenades obj;

   coordenades *o;
                  /*
   //COUT DELS PUNTS DE TOT EL CONTORN!!!!!!!!!!!!!
   cout << endl << "TOTS ELS PUNTS DEL CONTORN" << endl;
   o=totContorn->first();
   while(o){
      cout <<  sqrt (pow (o->X-imatge.dw->centreX, 2.0) + pow(o->Y-imatge.dw->centreY, 2.0)) << "   ";
      o=totContorn->next();
   }
   cout << endl;         
                */
   vector+= "<BR>PUNTS ESCOLLITS DEL CONTORN: <BR>";

   obj=totContorn->first();
   while (obj.X != -1){                          //Escriu coordenades
      vector+= "(" + a.setNum (obj.X) + ", ";
      vector+=a.setNum (obj.Y) + ")  ";
      obj=totContorn->next();
      }

   vector+= "<BR><BR>";

   if (descriptZ.metode>2){   //si escollim lopcio equidistants
      vector+="PUNTS EQUIDISTANTS DEL CONTORN: <BR>";
      for (int i=0;i<descriptZ.plenspreFD;i++){
        vector+= "(" + a.setNum (v_mostres[i].x) + ", ";
         vector+=a.setNum (fabs(v_mostres[i].y)) + ")  ";
/*///////////////////
         if (v_mostres[i].x - imatge.dw->centreX>0)
            cout << "+" << v_mostres[i].x - imatge.dw->centreX;
         else
            cout << v_mostres[i].x - imatge.dw->centreX;
         

         if (v_mostres[i].y + imatge.dw->centreY>0)
            cout << "+" << v_mostres[i].y + imatge.dw->centreY <<"i, ";
         else
            cout << v_mostres[i].y + imatge.dw->centreY <<"i, ";
*/////////////////////////         
         }
      vector+= "<BR><BR>";
   }
//             cout << endl;
   vector+="          TOTAL PUNTS=" + a.setNum(descriptZ.plenspreFD);
   vector+= "<BR>";

//   vector += "z ( t ): ";


   if (descriptZ.metode==1 || descriptZ.metode==3){  //hem fet complexes
/*      for (int i=0;i<descriptZ.plenspreFD;i++){
         vector+= "(" + a.setNum(punter_a_Z[i].x) + ", ";
         vector+= a.setNum(punter_a_Z[i].y) + "j) ";
         }

      vector+="<BR>";
      vector+="<BR>";
      vector+="PREDESCRIPTORS DE FOURIER";

      for (int i=0; i<descriptZ.plenspreFD; i++){   //printa els descriptors de fourier
         vector+= "(" + a.setNum(descriptZ.preFD[i].x) + ", ";
         vector+= a.setNum(descriptZ.preFD[i].y) + ") ";
         }
*/         
      vector+="<BR>";
      vector+="<BR>";
      
      vector+="DESCRIPTORS DE FOURIER";
      vector+= "<BR>";

      for (int i=0; i<descriptZ.plensFD; i++){   //printa els descriptors de fourier
         vector+= a.setNum(descriptZ.FD[i]) + ", ";
      }
      vector+="<BR>";
      
   }      
   else{ //NO complexe
/*      for (int i=0;i<descriptZ.plensFD;i++)
         vector+= a.setNum(punter_a_Z[i].x) + "   ";
   

   vector+="<BR>";
   vector+="<BR>";
   vector+="PREDESCRIPTORS DE FOURIER";
   vector+= "<BR>";


   for (int i=0; i<descriptZ.plenspreFD; i++){
      vector+= "(" + a.setNum(descriptZ.preFD[i].x) + ", ";
      vector+= a.setNum(descriptZ.preFD[i].y) + ") ";
   }
*/

   
      vector+="<BR>";
   vector+="<BR>";
   vector+="DESCRIPTORS DE FOURIER";
   vector+= "<BR>";

//   int meitat=descriptZ.plensFD/2;
   for (int i=0; i<descriptZ.plensFD; i++)
      vector+=a.setNum(descriptZ.FD[i]) + "  ";
   }
   
   vector += "<BR>";
   vector += "FOURIER UNIC: ";
   vector += a.setNum(resultat) + "<BR>";
   
//   vector +="Centre de masses: (" + a.setNum(imatge.dw->centreX);
//   vector += ", " + a.setNum(imatge.dw->centreY) + ")";


   return (vector);
}

//Fa la transformada discreta de fourier
void Fourier::DescriptorsFourier (){
   int i,j;
   double calcul;

   if (descriptZ.metode==1 || descriptZ.metode==3)    //els preFD son complexes
      descriptZ.plensFD=descriptZ.plenspreFD;  //tenim tants preFD com FD
   else   //els preFD no son complexes
      descriptZ.plensFD=descriptZ.plenspreFD/2;

   descriptZ.preFD = new complexe[descriptZ.plenspreFD];

   for (i=0; i<descriptZ.plenspreFD; i++){  //Inicialitzem a zero l'espai reservat
      descriptZ.preFD[i].x=0;
      descriptZ.preFD[i].y=0;
   }


   for (i=0; i < descriptZ.plenspreFD; i++){
      for (j=0; j < descriptZ.plenspreFD; j++){
         calcul=(2 * M_PI * j *i) / descriptZ.plenspreFD;
         descriptZ.preFD[i].x+= punter_a_Z[j].x * cos(calcul) + punter_a_Z[j].y * sin(calcul);
//         cout << "acumulacio " << descriptZ.preFD[i].x << "  multCOS " << punter_a_Z[j].x * cos(calcul) << "  multSIN " << punter_a_Z[j].y * sin(calcul) <<endl;
         descriptZ.preFD[i].y+= punter_a_Z[j].y * cos(calcul) - punter_a_Z[j].x * sin(calcul);
      }

//      cout << i << "    preFDx  " << descriptZ.preFD[i].x << "   preFDy   " << descriptZ.preFD[i].y << "   punterX  " << punter_a_Z[i].x << "   punterY   " << punter_a_Z[i].y << "   modul   " << sqrt (pow(punter_a_Z[i].x,2.0) + pow(punter_a_Z[i].y,2.0)) << endl;
      
      descriptZ.preFD[i].x=descriptZ.preFD[i].x / (double)descriptZ.plenspreFD;
      descriptZ.preFD[i].y=descriptZ.preFD[i].y / (double)descriptZ.plenspreFD;
   }
} 


void Fourier::escala (){
   int i, j;
   
//   descriptZ.plensFD=descriptZ.plenspreFD;


   descriptZ.FD = new double[descriptZ.plenspreFD];
   
   if (descriptZ.metode==1 || descriptZ.metode==3){   //complex coordinates
      for (i=0; i < descriptZ.plensFD; i++){
         descriptZ.FD[i]= sqrt (pow(descriptZ.preFD[i].x, 2.0) + pow(descriptZ.preFD[i].y,2.0));
//                  cout << "x  " << descriptZ.FD[i] << endl;
      }
                                                                             

      for (i=2; i<descriptZ.plensFD;i++)
         descriptZ.FD[i]=descriptZ.FD[i]/descriptZ.FD[1];

      descriptZ.FD[0]=0;
      descriptZ.FD[1]=1;
         

   }
   else{ //if (descriptZ.metode!=1){      //centroid distance
      for (i=0; i < descriptZ.plenspreFD; i++)           {
         descriptZ.FD[i]= sqrt (pow(descriptZ.preFD[i].x, 2.0) + pow(descriptZ.preFD[i].y,2.0));
//         cout << "x  " << descriptZ.FD[i] << endl;
      }
                                                

      for (i=1; i<descriptZ.plenspreFD;i++){
         descriptZ.FD[i]=descriptZ.FD[i]/descriptZ.FD[0];
//         cout << "x  " << descriptZ.FD[i] << endl;
      }
      descriptZ.FD[0]=1;
   }
}

//Uneix tots els descriptors per fer-ne un unic descriptor
void Fourier::DescriptorUnic (){
   double numerador=0.0, denominador=0.0;
   int k, max, a, i, inicia;
   resultat=0.0;

   
   if (descriptZ.metode==2 || descriptZ.metode==4){ //Centroid distance

      if ((descriptZ.plensFD % 2)==0)    //ens cal tenir un numero parell de valors
         max=descriptZ.plensFD;           //aquest comprobacio no te sentit si fem fourier equidistant
      else{                                         //pq segur k tenim un numero d mostres parell
         max=descriptZ.plensFD-1;
         cout << "ens hem carregat l'ultim descriptor per tenir-ne un numero parell" << endl;
      }
      
     
      nfd = new double[max];
      
                   /*
      for (i=0; i<max*2;i++)
         cout << i << "    " << descriptZ.FD[i] << endl;
         
      for (i=0; i<max-1; i++) {    //Busquem NFD(k) part negativa
         nfd[i]=descriptZ.FD[i+max+1];
         cout << "<<" << nfd[i] << endl;
      }          
         */                                                                //en tenim plens la meitat menys un
//      nfd[0]=0.0;

      inicia=max-1;

      for (i=0; i<=inicia; i++)   //Valor abolut de NFD(k)
         if (nfd[i]<0)
            nfd[i]*=(-1);
            
    
      for (i=1;i<max;i++){
         numerador=numerador+(double)fabs(descriptZ.FD[i]) / (double) i;
         denominador=denominador+fabs(descriptZ.FD[i]);
//         cout << i <<"    " << fabs(descriptZ.FD[i]) <<  "     " <<numerador << "    " << denominador << endl;         
      }
      
  

                              /*
                        
      for (i=inicia; i>0; i--){
         numerador=numerador+(double)nfd[inicia-i]/(double) i;
         denominador=denominador+nfd[inicia-i];
         cout << inicia-i <<"    " << nfd[inicia-i]<< "     " <<numerador << "    " << denominador << endl;
      }                         */
  
   }
   else{    //complex coordinates  
      if ((descriptZ.plensFD % 2)==0)    //ens cal tenir un numero parell de valors
         max=descriptZ.plensFD;
      else{
         max=descriptZ.plensFD-1;
         cout << "ens hem carregat l'ultim descriptor per tenir-ne un numero parell" << endl;
      }
      
      nfd = new double[max];


      for (i=0; i<max/2-1; i++)     //Busquem NFD(k) part negativa
         nfd[i]=descriptZ.FD[i+max/2+1];

      a=1;
      for (i=max/2;i<max;i++)       //Busquem els NFD(k) part positiva
         nfd[i]=descriptZ.FD[a++];

      nfd[max/2-1]=0;
      

      for (i=0; i<max; i++)   //Valor abolut de NFD(k)
         if (nfd[i]<0)
            nfd[i]*=(-1);


      a=0;
      for (i=-max/2+1;i <=max/2; i++){
         if (i<0)
            k=i*(-1);
         if (k!=0){
            numerador=numerador+(double)nfd[a]/(double)k;
            denominador=denominador+nfd[a++];
         }
      }
   
   }


   resultat=numerador/denominador;
}


void Fourier::AntiTransformada (){
      double calcul;
      int i, j;
      FILE *f;
      f=fopen ("/home/galco/Desktop/d.txt","w");
      complexe taula[descriptZ.plenspreFD];

      for (i=0; i< descriptZ.plenspreFD;i++){
         taula[i].x=0;
         taula[i].y=0;
      }
      
//      if (descriptZ.metode==1){
         for (i=0; i < descriptZ.plenspreFD; i++){
            for (j=0; j < descriptZ.plenspreFD; j++){
               calcul=(2 * M_PI * j * i) / descriptZ.plenspreFD;
               taula[i].y+= descriptZ.preFD[j].x * cos(calcul) + descriptZ.preFD[j].y * sin(calcul);
               taula[i].x+= descriptZ.preFD[j].y * cos(calcul) - descriptZ.preFD[j].x * sin(calcul);
            }
         }

         for (i=0; i < descriptZ.plenspreFD; i++)
         fprintf (f, "(%-10f    %-10f)     (%-10f     %-10f)\n", punter_a_Z[i].x, punter_a_Z[i].y, taula[i].x, taula[i].y);
     
     fclose (f);
}
      
void Fourier::EquidistaPixels (Imatge& im, double total_mostres){
   int i=1;
   complexe acumula, tmp;
   double mostres, pas, passosP, acumulaModul, afegir;
   ModulFase actual;
   coordenades *el;

   v_mostres = new complexe[total_mostres];

   acumula.x=0.0;
   acumula.y=0.0;
   acumulaModul=0.0;

   pas=im.contorn->perimetre / total_mostres;

   actual=*im.contornPolar->first();
   mostres=actual.modul;


   v_mostres[0].x=(double)im.dw->llista->first()->X; //Ho iniciem am el primer punt pq sino traslladem la figura al (0,0)
   v_mostres[0].y=(double)im.dw->llista->first()->Y; //I em puteja perqu`e a l'hora de calcular la dist`ancia respecte el centre no es el mateix

   passosP=pas;
//   int t;
//   t=im.contornPolar->count();
//   cout << "t  " << t << endl;
   
   while (i < total_mostres){
      if (mostres >= passosP){
         afegir=passosP-acumulaModul;
         tmp=passa_a_complexe (afegir, actual.fase);         
         v_mostres[i].x=v_mostres[0].x+acumula.x+tmp.x;    //guardo els vectors respecte l'origen, no respecte l'anterior
         v_mostres[i].y=v_mostres[0].y+acumula.y+tmp.y;
//   cout << "if    " << v_mostres[i].x << "       " <<  v_mostres[i].y  << endl;
         i++;
         passosP+=pas;
      }
      else{
         tmp=passa_a_complexe (actual.modul, actual.fase);
         acumula.x+=tmp.x;
         acumula.y+=tmp.y;
         acumulaModul+=actual.modul;
         actual=*im.contornPolar->next();
         mostres+=actual.modul;
      }

//      cout << "lalala " << i << "    " << v_mostres[i].x << "       " <<  v_mostres[i].y  << "    " << actual.modul << "    " << actual.fase << endl;
   }

   //Copiem els punts equidistants a la llista del dw per ser mostrats com a creuetes vermelles
   im.dw->llista = new ContornL;
   for (i=0;i<total_mostres;i++){
      el=new coordenades;
      el->X=(int)round(v_mostres[i].x);
      el->Y=(int)round(v_mostres[i].y);
      im.dw->llista->append (el);
   }

   el=new coordenades;
   el->X=-1;
   
   im.dw->llista->append (el);
}

complexe Fourier::passa_a_polar (double x, double y){
   complexe a;

   a.x = sqrt(pow(x,2.0) + pow(y,2.0));
   a.y = tan (y/x);

   return a;
}

complexe Fourier::passa_a_complexe (double modul, double fase){
   complexe a;
   double resCos, resSin;

  if (fase == 1.5707963267948966){
     resCos=0;
     resSin=1;
  }
  else if (fase == -1.5707963267948966){
     resCos=0;
     resSin=-1;
  }
  else if (fase == 3.1415926535897931){
     resCos=-1;
     resSin=0;
  }
  else if (fase == 0.0){
     resCos=1;
     resSin=0;
  }
/*  else if (fase == 2.35619449019234492){
     resCos=-0.7071067669;
     resSin=0.7071067669;
  }
  else if (fase == -2.35619449019234492){
     resCos=-0.7071067669;
     resSin=-0.7071067669;
  }
  else if (fase == 0.785398163397448309615){
     resCos=0.7071067811865475244;
     resSin=0.7071067811865475244;
  }
  else if (fase == -0.785398163397448309615){
     resCos=0.7071067811865475244;
     resSin=-0.7071067811865475244;
  }                */
  else{
      resCos=cos (fase);
      resSin=sin (fase);
  }

                 
   a.x =(double) modul * resCos;
   a.y =(double) -modul * resSin;

   return a;
}

//Transformada rapida de fourier
void Fourier::FFT(int dir,int m)
{
   long n,i,i1,j,k,i2,l,l1,l2;
   double c1,c2,tx,ty,t1,t2,u1,u2,z;


    if (descriptZ.metode==1 || descriptZ.metode==3)    //els preFD son complexes
      descriptZ.plensFD=descriptZ.plenspreFD;  //tenim tants preFD com FD
   else   //els preFD no son complexes
      descriptZ.plensFD=descriptZ.plenspreFD/2;

   descriptZ.preFD = new complexe[descriptZ.plenspreFD];

   /* Calculate the number of points */
   n = 1;
   for (i=0;i<m;i++)
      n *= 2;


   for (i=0;i<n;i++){
	   descriptZ.preFD[i].x=punter_a_Z[i].x;
	   descriptZ.preFD[i].y=punter_a_Z[i].y;
   }

   /* Do the bit reversal */
   i2 = n >> 1;
   j = 0;
   for (i=0;i<n-1;i++) {
      if (i < j) {
         tx = descriptZ.preFD[i].x;
         ty = descriptZ.preFD[i].y;
         descriptZ.preFD[i].x = descriptZ.preFD[j].x;
         descriptZ.preFD[i].y = descriptZ.preFD[j].y;
         descriptZ.preFD[j].x = tx;
         descriptZ.preFD[j].y = ty;
      }
      k = i2;
      while (k <= j) {
         j -= k;
         k >>= 1;
      }
      j += k;
   }

   /* Compute the FFT */
   c1 = -1.0;
   c2 = 0.0;
   l2 = 1;
   for (l=0;l<m;l++) {
      l1 = l2;
      l2 <<= 1;
      u1 = 1.0;
      u2 = 0.0;
      for (j=0;j<l1;j++) {
         for (i=j;i<n;i+=l2) {
            i1 = i + l1;
            t1 = u1 * descriptZ.preFD[i1].x - u2 * descriptZ.preFD[i1].y;
            t2 = u1 * descriptZ.preFD[i1].y + u2 * descriptZ.preFD[i1].x;
            descriptZ.preFD[i1].x = descriptZ.preFD[i].x - t1;
            descriptZ.preFD[i1].y = descriptZ.preFD[i].y - t2;
            descriptZ.preFD[i].x += t1;
            descriptZ.preFD[i].y += t2;
         }
         z =  u1 * c1 - u2 * c2;
         u2 = u1 * c2 + u2 * c1;
         u1 = z;
      }
      c2 = sqrt((1.0 - c1) / 2.0);
      if (dir == 1)
         c2 = -c2;
      c1 = sqrt((1.0 + c1) / 2.0);
   }

   /* Scaling for forward transform */
   if (dir == 1) {
      for (i=0;i<n;i++) {
         descriptZ.preFD[i].x /= n;
         descriptZ.preFD[i].y /= n;
      }
   }

   return(TRUE);


}


//Codi tret d'internet  fa la transformada discreta de fourier igual k el meu (Descriptors de fourier)
void Fourier::DFT()
{
   long i,k;
   double arg;
   double cosarg,sinarg;

   if (descriptZ.metode==1 || descriptZ.metode==3)    //els preFD son complexes
      descriptZ.plensFD=descriptZ.plenspreFD;  //tenim tants preFD com FD
   else   //els preFD no son complexes
      descriptZ.plensFD=descriptZ.plenspreFD/2;

   descriptZ.preFD = new complexe[descriptZ.plenspreFD];

   for (i=0;i<descriptZ.plenspreFD;i++) {
      descriptZ.preFD[i].x = 0;
      descriptZ.preFD[i].y = 0;
      arg =  (-1) * 2.0 * 3.141592654 * (double)i / (double)descriptZ.plenspreFD;    
      for (k=0;k<descriptZ.plenspreFD;k++) {
         cosarg = cos(k * arg);
         sinarg = sin(k * arg);
         descriptZ.preFD[i].x += (punter_a_Z[k].x * cosarg - punter_a_Z[k].y * sinarg);
         descriptZ.preFD[i].y += (punter_a_Z[k].x * sinarg + punter_a_Z[k].y * cosarg);
      }
   }


   for (i=0;i<descriptZ.plenspreFD;i++) {
      descriptZ.preFD[i].x = descriptZ.preFD[i].x / (double)descriptZ.plenspreFD;
      descriptZ.preFD[i].y = descriptZ.preFD[i].y / (double)descriptZ.plenspreFD;
            cout << descriptZ.preFD[i].x << "   " << descriptZ.preFD[i].y << endl;
   }

}


QString Fourier::MostraComparacio(Fourier &f1){
   int i=0, tamany;
   QString vector, a;
   double *resta, resultat=0.0;

   resta = new double[descriptZ.plensFD];

   vector+="<BR>";
   vector+="<BR>";

      

   if (descriptZ.metode>2) //Punts equidistants
      for (i=0;i<descriptZ.plensFD;i++)
         resta[i]= descriptZ.FD[i] - f1.descriptZ.FD[i];


      vector+="COMPARACIO ENTRE DESCRIPTORS";
      vector+="<BR>";
      vector+="<BR>";
         
      if (descriptZ.plensFD>f1.descriptZ.plensFD)
         tamany=descriptZ.plensFD;
      else
         tamany=f1.descriptZ.plensFD;
         
      for (i=0;i<tamany;i++)
         vector+= a.setNum(resta[i]) + "   ";

      vector+="<BR>";
      vector+="<BR>";
      
      vector+="DESCRIPTORS 1a FIGURA";

      vector+="<BR>";
      vector+="<BR>";

      for (i=0;i<tamany;i++)
         vector+= a.setNum(descriptZ.FD[i]) + "   ";

      vector+="<BR>";
      vector+="<BR>";
      
      vector+="DESCRIPTORS 2a FIGURA";
      vector+="<BR>";
      vector+="<BR>";

      for (i=0;i<tamany;i++)
         vector+= a.setNum(f1.descriptZ.FD[i]) + "   ";
      
      for (i=0;i<tamany;i++)
         resultat+=pow (resta[i], 2.0);

      resultat=sqrt (resultat);

      vector+="<BR>";
      vector+="<BR>";
      vector+="DISTANCIA EUCLIDIANA";
      vector+="<BR>";
      vector+="<BR>";
      vector+=a.setNum (resultat);
      
   return (vector);
   }




