/***************************************************************************
                          caractdoc.cpp  -  description
                             -------------------
    begin                : jue nov  6 12:49:24 CET 2003
    copyright            : (C) 2003 by projecte
    email                : projecte@localhost.localdomain
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qdir.h>
#include <qfileinfo.h>
#include <qwidget.h>
#include <qmsgbox.h>
#include <qfiledialog.h>


// application specific includes
#include "caractdoc.h"
#include "caract.h"
#include "caractview.h"


CaractDoc::CaractDoc()
{
  pViewList = new QList<CaractView>;
  pViewList->setAutoDelete(false);
}

CaractDoc::~CaractDoc()
{
  delete pViewList;
}

void CaractDoc::addView(CaractView *view)
{
  pViewList->append(view);
	changedViewList();
}

void CaractDoc::removeView(CaractView *view)
{
	  pViewList->remove(view);
	  if(!pViewList->isEmpty())
			changedViewList();
		else
			deleteContents();
}

void CaractDoc::changedViewList(){	
	
	CaractView *w;
	if((int)pViewList->count() == 1){
  	w=pViewList->first();
  	w->setCaption(m_title);
	}
	else{
		int i;
    for( i=1,w=pViewList->first(); w!=0; i++, w=pViewList->next())
  		w->setCaption(QString(m_title+":%1").arg(i));	
	}
}

bool CaractDoc::isLastView() {
  return ((int) pViewList->count() == 1);
}


void CaractDoc::updateAllViews(CaractView *sender)
{
  CaractView *w;
  for(w=pViewList->first(); w!=0; w=pViewList->next())
  {
     w->update(sender);
  }

}

void CaractDoc::setPathName(const QString &name)
{
  m_filename=name;
	m_title=QFileInfo(name).fileName();
}

const QString& CaractDoc::pathName() const
{
  return m_filename;
}

void CaractDoc::setTitle(const QString &title)
{
  m_title=title;
}

const QString &CaractDoc::title() const
{
  return m_title;
}


void CaractDoc::closeDocument()
{
  CaractView *w;
  if(!isLastView())
  {
    for(w=pViewList->first(); w!=0; w=pViewList->next())
    {
   	 	if(!w->close())
 				break;
    }
	}
  if(isLastView())
  {
  	w=pViewList->first();
  	w->close();
  }
}

bool CaractDoc::newDocument()
{
  /////////////////////////////////////////////////
  // TODO: Add your document initialization code here
  /////////////////////////////////////////////////
  modified=false;
  return true;
}

bool CaractDoc::openDocument(const QString &filename, const char *format /*=0*/)
{

  float k;
  int n;
  QString titol("(%) ");
  /////////////////////////////////////////////////
  // TODO: Add your document opening code here
  /////////////////////////////////////////////////

  imatge.openImatge(filename);
  modified=false;
  m_filename=filename;
//	m_title=filename;
  // codi perque escrigui l'escala a la capcalera de la finestra
  k=((float)imatge.zm.w/imatge.imatge.dimX)*100;
  n=k;
  while(n>0){
    titol.insert(1,(n%10)+48);
    n=n/10;
  }
  m_title = titol+filename;

  return true;
}

bool CaractDoc::saveDocument(const QString &filename, const char *format /*=0*/)
{
	QFile f( filename+".pgm" );

  	if ( !f.open( IO_WriteOnly ) )
  		return false;

    QTextStream stream(&f);
    QString amplada;
    QString alcada;
    QString estat;
    QString valor;
    int i,j;

      // Codi per guardar imatges MAMO sobre PGM
      amplada.setNum(imatge.imatge.dimX);
      alcada.setNum(imatge.imatge.dimY);
      estat.setNum(imatge.estat);


      stream << "P5 #MAMO contents=" << estat << "\n";
      stream << amplada << "\n";
      stream << alcada << "\n";
      stream << "255" << "\n";  // profunditat
      // guarda la info de la imatge
      for(j=0;j<imatge.imatge.dimY;j++){
        for(i=0;i<imatge.imatge.dimX;i++){
            f.putch(imatge.imatge.data(i,j));
        }
      }

    f.close();

  modified=false;
  m_filename=filename;
	m_title=QFileInfo(f).fileName();
  return true;
}

void CaractDoc::deleteContents()
{
  /////////////////////////////////////////////////
  // TODO: Add implementation to delete the document contents
  /////////////////////////////////////////////////

}

bool CaractDoc::canCloseFrame(CaractView* pFrame)
{
	if(!isLastView())
		return true;
		
	bool ret=false;
  if(isModified())
  {
		QString saveName;
  	switch(QMessageBox::information(pFrame, title(), tr("The current file has been modified.\n"
                          "Do you want to save it?"),QMessageBox::Yes, QMessageBox::No, QMessageBox::Cancel ))
    {
			case QMessageBox::Yes:
				if(title().contains(tr("Untitled")))
				{
					saveName=QFileDialog::getSaveFileName(0, 0, pFrame);
                                          if(saveName.isEmpty())
          	                                return false;
				}
				else
					saveName=pathName();

				if(!saveDocument(saveName))
				{
 					switch(QMessageBox::critical(pFrame, tr("I/O Error !"), tr("Could not save the current document !\n"
																												"Close anyway ?"),QMessageBox::Yes ,QMessageBox::No))

 					{
 						case QMessageBox::Yes:
 							ret=true;
 						case QMessageBox::No:
 							ret=false;
 					}
				}
				else
					ret=true;
				break;
			case QMessageBox::No:
				ret=true;
				break;
			case QMessageBox::Cancel:
			default:
				ret=false;
				break;
		}
	}
	else
		ret=true;

	return ret;
}
