/***************************************************************************
                          brent.cpp  -  description
                             -------------------
    begin                : Wed Sep 25 2002
    copyright            : (C) 2002 by Robert Marti
    email                : robertmarti@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "brent.h"
#include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include "caract/rmlib/rmutils/rmutils.h"

Brent::Brent(){
}
Brent::~Brent(){
}

inline void shift1(double& a, double& b, double& c, const double& d)
{
  a=b; b=c; c=d;
}

inline void shift2(float& a, float& b, float& c, const float& d)
{
  a=b; b=c; c=d;
}

double Brent::solve( double ax, double bx, double cx,
		 rmFunc1D & F, double tol,	 double &xmin )
{
	const int ITMAX = 100 ;
	const double  CGOLD = 0.3819660 , ZEPS = 1.0e-10 ;

	int iter;
	double a,b,d,etemp,fu,fv,fw,fx,p,q,r,tol1,tol2,u,v,w,x,xm;
	double e=0.0;

	a=(ax < cx ? ax : cx);
	b=(ax > cx ? ax : cx);
	x=w=v=bx;
	fw=fv=fx=F(x);
	for (iter=1;iter<=ITMAX;iter++) {
		xm=0.5*(a+b);
		tol2=2.0*(tol1=tol*fabs(x)+ZEPS);
		if (fabs(x-xm) <= (tol2-0.5*(b-a))) {
			xmin=x;
			return fx;
		}
		if (fabs(e) > tol1) {
			r=(x-w)*(fx-fv);
			q=(x-v)*(fx-fw);
			p=(x-v)*q-(x-w)*r;
			q=2.0*(q-r);
			if (q > 0.0) p = -p;
			q=fabs(q);
			etemp=e;
			e=d;
			if (fabs(p) >= fabs(0.5*q*etemp) || p <= q*(a-x) || p >= q*(b-x))
				d=CGOLD*(e=(x >= xm ? a-x : b-x));
			else {
				d=p/q;
				u=x+d;
				if (u-a < tol2 || b-u < tol2)
					d= rmUtils::sign(tol1,xm-x);
			}
		} else {
			d=CGOLD*(e=(x >= xm ? a-x : b-x));
		}
		u=(fabs(d) >= tol1 ? x+d : x+ rmUtils::sign(tol1,d));
		fu=F(u);
		if (fu <= fx) {
			if (u >= x) a=x; else b=x;
			shift1(v,w,x,u);
			shift1(fv,fw,fx,fu);
		} else {
			if (u < x) a=u; else b=u;
			if (fu <= fw || w == x) {
				v=w;
				w=u;
				fv=fw;
				fw=fu;
			} else if (fu <= fv || v == x || v == w) {
				v=u;
				fv=fu;
			}
		}
	}
	cerr << "Too many iterations in Brent" << endl ;
	abort();
	xmin=x;
	return fx;
}

