// DICOM format reading files
//    Robert Marti 2001 
#ifndef rmDICOM_H
#define rmDICOM_H

#include "caract/rmlib/rmmatrix/rmmatrix2d.h"
//#include "rmlib/rmimage/rmimage.h"
#include <stdio.h>
#include <iostream.h>
#include <stdlib.h>

#define DCM_OK 0
#define DCM_ERROR 1
#define TRUE 1
#define FALSE 0

typedef short imdata; // abans no era unsigned

typedef enum { 
	TNONE, TSTRING, TSTRINT, TFLOAT, TDOUBLE,
	TLONG, TULONG, TSHORT, TUSHORT, TSEQ
	} dicom_types;

typedef unsigned char uchar;
typedef unsigned int uint;

/**
	DICOM file format  class
  *@author Robert Marti
  */

class rmDICOM {

private:
	int prused; 	// true if pre-header is used (!=0)
	int lE;	  	// true if little endian 
	int explVR;	// true if explicit VR is being used.
	uint dimX, dimY;
	long MaxValue;
	uint bpp;
	int finished;
	uchar doPrint;
	double voxelSize[3];
	double sliceSpacing;

public:

	/** Default constructor*/
	rmDICOM();
	/** Constructor*/
	rmDICOM(int);
	/** Read DICOM file. Input Parameters:
		- filename (in)

	Returns:
		- Image dimensions (dx, dy)
		- Number of bits
		- Maximum Value (Mval)
		- Image data matrix
		- Voxel Size vs
		- Slice spacing ss*/
	void readFile(char * filename,int &dX, int &dY, int &bits, int &MVal,
		rmMatrix2d<imdata> &data, double *vs, double ss);
	/** Read Dicom header from a file defined by fd*/
	int readHeader(FILE * fd);
	/** Print some DICOM info*/
	void Print (int);

private:

	void readInfo (FILE * fd,rmMatrix2d<imdata> &data);
	void checkGroup (int group, int element, int len, char * svr,
                	dicom_types tt, FILE * fd,rmMatrix2d<imdata> &data);
	int explicitVR(char * vr, dicom_types & tt);
	void setEndian();
	imdata readPixel(FILE *);
	int read16( FILE * fd) ;
	unsigned int readu16( FILE * fd); 
	long read32 (FILE * fd);
	unsigned long readu32 (FILE * fd);
};
#endif
