/***************************************************************************
                          rmhistogram.cpp  -  description
                             -------------------
    begin                : Tue Jul 2 2002
    copyright            : (C) 2002 by Robert Marti
    email                : robertmarti@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "rmhistogram.h"
#include <math.h>

rmHistogram::rmHistogram()
{

}


rmHistogram::rmHistogram(int NoInt)
{
	init(NoInt);
}

void rmHistogram::init(int NoInt)
{
	int i;

	N=NoInt;
	data=new long int [N];
	datan=new double[N];
	for (i=0;i<N;i++)
	{
		data[i]=0;
		datan[i]=0;
	}
	range = new int [2];
	range[0]= 0;
	range[1]= 0;
	peak=0;
}

rmHistogram::~rmHistogram()
{
	delete data;
	delete datan;
	delete range;
}

void rmHistogram::create(const rmImage & image)
{
	long int i,j;

	for (i=0;i<N;i++)
		data[i]=0;	
	// Generate histogram
	range[0]=N;
	range[1]=0;
	peak=0;	
	for (i=0;i<image.dimX;i++)
	  for (j=0;j<image.dimY;j++)
	  {
			if (image.data(i,j)>range[1]) range[1]=image.data(i,j);
			if (image.data(i,j)<range[0]) range[0]=image.data(i,j);
			data[image.data(i,j)]++;
			if (data[image.data(i,j)]>peak) peak=data[image.data(i,j)];		
	  }
	NP= image.dimX*image.dimY;
	normaliseHistogram();
}	

void rmHistogram::create(const rmImage3d &image)
{
	int i,j,k;

	for (i=0;i<N;i++)
    	data[i]=0;	
	// Generate histogram
	range[0]=N;
	range[1]=0;
	peak=0;	
	for (i=0;i<image.dimX;i++)
	  for (j=0;j<image.dimY;j++)
	    for (k=0;k<image.dimZ;k++) {
		  	//if (image(i,j,k)>0) // not using low grey-levels
				data[image(i,j,k)]++;
				if (data[image(i,j,k)]>peak)
		 			peak=data[image(i,j,k)];		
				if (image(i,j,k)>range[1])
					range[1]=image(i,j,k);
				if (image(i,j,k)<range[0])
					range[0]=image(i,j,k);
	}
	cout << "b4 norm " << range[0] << " "<< range[1]<< " "<< peak;
	NP= image.dimX*image.dimY*image.dimZ;
	normaliseHistogram();
	cout <<" create" << endl;
}	


long int rmHistogram::readValue(int i)
{	
	return data[i];
}

void rmHistogram::readRange(long *r)
{
	r[0]=(long)range[0];
	r[1]=(long)range[1];
}

void rmHistogram::readRange(int r[2])
{
	r[0]=range[0];
	r[1]=range[1];
}

long int rmHistogram::readPeak()
{
	return peak;
}


// Create Normalised histogram 0..1
void rmHistogram::normaliseHistogram()
{
	int i;
	
	for (i=0;i<N;i++)
	{
		datan[i]=double(data[i])/double(NP);	
	}
}

double rmHistogram::readNorm(int i) {
	return datan[i];
}

void rmHistogram::equalise(rmImage &im) {
	float alpha;
	alpha=float(255)/float(NP);
	long *aux=new long [N];
	int i,j;

	cout << alpha << endl;
	aux[0]= long(alpha*data[0]);
	for (i=1;i<N;i++)
		aux[i]=long((aux[i-1]+alpha*data[i]));
	for (i=0;i<im.dimX;i++)
		for (j=0;j<im.dimY;j++)
 	   im.data(i,j)=aux[im.data(i,j)];
	delete aux;
}


void rmHistogram::stretch(rmImage &im) {
	float denom=range[1]-range[0];
	int i,j;

	for (i=0;i<im.dimX;i++)
		for (j=0;j<im.dimY;j++)
			im.data(i,j)= (byte)rint(255*(im.data(i,j)-range[0])/denom );
}


void rmHistogram::getImage (rmImage &im) {
    int i,j;
	
	im.resize(N+100,250);
	im.data.clear(255);
	
	for (i=50;i<im.dimX-50;i++)  // for each grey level
	{
		for (j=im.dimY-25;j>im.dimY-(26+200*data[i-50]/peak);j--)
		{
			im.data(i,j)= 0;
		}
	}
	//im.Save("histo.byte");
}
	
