/***************************************************************************
                          rmhistogram2d.cpp  -  description
                             -------------------
    begin                : Thu Jul 4 2002
    copyright            : (C) 2002 by Robert Marti
    email                : robertmarti@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "rmhistogram2d.h"

rmHistogram2d::rmHistogram2d(int NoInt)
{
	init(NoInt);
	clear();
}

rmHistogram2d::rmHistogram2d()
{
}

void rmHistogram2d::init(int NoInt)
{
	int i;

	N=NoInt;
	datan=new double *[N];
	for (i=0;i<N;i++)
	{
		datan[i]=new double[N];
	}
	div=256/N;
}

void rmHistogram2d::clear()
{
	int i,j;
	for (i=0;i<N;i++)
		for (j=0;j<N;j++)
		{
			datan[i][j]=0.0;
		}
	range[0]= 0;
	range[1]= 0;
	peak=0;
	NP=0;
}

rmHistogram2d::~rmHistogram2d()
{
	int i;

	for (i=0;i<N;i++)
	{
		delete[] datan[i];
	}
	delete[] datan;
}

void rmHistogram2d::create(const rmImage & image1, const rmImage & image2)
{
	int i,j;
	
	int dx,dy;
	if (image1.dimX>image2.dimX) dx = image2.dimX;
	else dx =image1.dimX;
 	if (image1.dimY>image2.dimY) dy = image2.dimY;
	else dy =image1.dimY;
	for (i=0;i<dx;i++)
		for (j=0;j<dy;j++){
			datan[image1.data(i,j)/div][image2.data(i,j)/div]++;
	}
	normalise();
}

void rmHistogram2d::create(const rmImage3d &image1,const rmImage3d &image2)
{
	int i,j,k;
	
	int dx,dy,dz;
	if (image1.dimX>image2.dimX) dx = image2.dimX;
	else dx =image1.dimX;
 	if (image1.dimY>image2.dimY) dy = image2.dimY;
    else dy =image1.dimY;
 	if (image1.dimZ>image2.dimZ) dz = image2.dimZ;
    else dz =image1.dimZ;

	for (i=0;i<dx;i++)
	  for (j=0;j<dy;j++)
	   	for (k=0;k<dz;k++) {
		    datan[image1(i,j,k)/div][image2(i,j,k)/div]++;
	}
	normalise();
}


void rmHistogram2d::normalise()
{
	int i,j;

	NP=0;
	for (i=0;i<N;i++)
		for (j=0;j<N;j++) {
			NP+=datan[i][j];
	}
	if (NP>0)
		for (i=0;i<N;i++)
			for (j=0;j<N;j++) {
				if (datan[i][j]>0) datan[i][j]=datan[i][j]/NP;
				if (datan[i][j]>peak)
				peak=datan[i][j];
			}
}
	

double rmHistogram2d::readNorm(int i,int j)
{	
	return datan[i][j];
}


void rmHistogram2d::add(int i, int j, double value)
{
	datan[i/div][j/div]+=value;
}

void rmHistogram2d::readRange(long int r[2])
{
	r[0]=range[0];
	r[1]=range[1];
}

double rmHistogram2d::readPeak()
{
	return peak;
}

void rmHistogram2d::createImage (rmImage &image)
{
	int i,j;

	image.resize(N,N);
	image.data.clear(0);

	for (i=0;i<N;i++)
		for (j=0;j<N;j++)
		{
			image.data(i,j)=byte(datan[i][j]*(double(N)/double(peak)));
	}
}

