// Image Class
//    Robert Marti 1999

//#include "Rad/LinFilt.h"
#include "rmimage.h"
#include <math.h>
#include "caract/rmlib/rmutils/rmutils.h"
//#include "ShapeMod/GaussFns.h"
//#include "ShapeMod/Random.h"
#include "caract/rmlib/rmmatrix/rmmatrix2d.h"
#include "caract/rmlib/rmimage/rmhistogram.h"
//#include "Rad/BlobOp.h"
#include <libgen.h>
#include <string.h>

#include "../../contornl.h"

#define  FTOI(a) ( (a) < 0 ? ((int)(a-0.5)) : ((int)(a+0.5)) )
#define SEVEN_SUPP

rmImage::rmImage()
{
	MaxValue=256;
	bits=8;
	voxelSize[0]=1.0;
	voxelSize[1]=1.0;
	voxelSize[2]=1.0;
	sliceSpacing=0.0;
}

rmImage::rmImage(char *filename)
{
	voxelSize[0]=1.0;
	voxelSize[1]=1.0;
	voxelSize[2]=1.0;
	sliceSpacing=0.0;
	read(filename);
}

/**
* Print Image info
**/
void rmImage::printInfo()
{
	cout <<"Dimensions X: "<< dimX << " Y:" << dimY << endl;
	cout <<"MaxValue "<< MaxValue;
	cout <<" Bits "<<bits << endl;
	cout <<"VoxelSize "<<voxelSize[0];
	cout <<" "<<voxelSize[1];
	cout <<" "<<voxelSize[2]<< endl;
	cout <<"Slice Spacing " <<sliceSpacing << endl;
}

/**
* Read Image from a file and store in rmImage object
*      Parameters: filename : filename to load image from
*
* 	This function determines which format (mit or byte) and
* 	reads the image.
**/
void rmImage::read(char *filename)
{
	format=findFormat(filename);
	switch (format)
	{
	case RM_DICOM:readDICOM(filename);break;
	case RM_MIT:	readMit(filename);break;
	case RM_BYTE:	readByte(filename);break;
  case RM_TIFF: readTiff(filename);break;
  case RM_RAW: readRaw(filename);break;
  case RM_PGM: readPgm(filename);break;
	case RM_NO_FORMAT:break;
	}
}

/**
* Find format of the image (.mit or .byte)
*      Parameters: filename : filename to load image from
*
*      This function determines which format (mit or byte) and
*      reads the image.
*/
int rmImage::findFormat(char * filename)
{
  if(strstr(filename,".pgm")!=NULL){
    return RM_PGM;
  }
  else{
	if (strstr(filename,".mit")!=NULL)
	{
		return RM_MIT;
	}
	else{
     if (strstr(filename,".raw")!=NULL || strstr(filename,".RAW")!=NULL)
     {
       return RM_RAW;
	   }
     else{

      if(strstr(filename,".tiff")!=NULL || strstr(filename,".tif")!=NULL){
      return RM_TIFF;
    }
    else{
  		if (strstr(filename,".byte")!=NULL)
	  	{
		  	return RM_BYTE;
		  }
		  else {
        if (!strstr(filename,".dcm")!=NULL)
	  	  {
		  	  return RM_NO_FORMAT;
		    }

			  FILE *fd = fopen(filename,"r");
			  if (dicom.readHeader(fd)== DCM_OK) {
				  fclose(fd);
          return RM_DICOM;
			  }
			  fclose(fd);
			  return RM_NO_FORMAT;
      }
		}
    }
	}
  }
}

/**
* Read a DICOM format Image from a file
*      Parameters: filename : filename to load image from
**/
void rmImage::readDICOM(char *filename)
{
	sliceSpacing = -1;
	cout << "DICOM image: "<<filename<< endl;
	dicom.readFile(filename,dimX,dimY,bits,MaxValue,data,voxelSize,
	sliceSpacing);
	//cout << "Info: ("<< dimX << ","<< dimY<< ") "<<
	//bits << " MAX "<< MaxValue << endl;
	printInfo();
}

/**
* Read a mit image from a file and store in a rmImage object
*      Parameters: filename : filename to load image from MIT format
**/
void rmImage::readMit(char *filename)
{
	FILE * fd;

	fd=fopen(filename,"r");

	cout << "MIT image: "<<filename<< endl;
	if (fd==NULL)	{
		error("Opening MIT Image");
	}
	else {
		readMitHeader(fd);
		switch (bits)
		{
		case 8:	readImage8(fd);
			break;
		case 12:readImage12(fd);
			break;
		case 16:readImage16(fd);
			break;
		}
	}
	fclose(fd);
}

/**
* Read MIT header
*      Parameters: fd : filename descriptor
*
*/
void rmImage::readMitHeader(FILE *fp)
{
	int low,high;

	low    = getc(fp);
  high   = getc(fp);
  ftype = low + (high * 256);

  low    = getc(fp);
  high   = getc(fp);
  bits  = low + (high * 256);

  low    = getc(fp);
  high   = getc(fp);
  dimX = low + (high * 256);

  low    = getc(fp);
  high   = getc(fp);
  dimY = low + (high * 256);
}

/**
* Read 8 bit Image
*      Parameters: fd : file descriptor
*/
void rmImage::readImage8(FILE *fd)
{
	int i,j;
  long n;
  MaxValue=256;

  data.resize(dimX,dimY);
  n=0;
  for (j=0;j<dimY;j++)
  	for (i=0;i<dimX;i++)
    	data(i,j)=getc(fd);



}

/**
* Read 8 bit Image (Y Reversed)
*      Parameters: fd : file descriptor
*/
void rmImage::readImage8R(FILE *fd)
{
	int i,j;
	long n;
	MaxValue=256;
 	data.resize(dimX,dimY);
 	n=0;
  for (j=0;j<dimY;j++)
  	for(i=0;i<dimX;i++)
    	data(i,dimY-1-j)=getc(fd);
}

/**
** Read 12 bit Image
**      Parameters: fd : filename descriptor
**
**/
void rmImage::readImage12(FILE *fd)
{
	readImage16(fd);
	MaxValue=4096;
}
/**
** Read 16 bit Image
**      Parameters: fd : filename descriptor
**
**/
void rmImage::readImage16(FILE *fd)
{
	int i,j;
	byte h,l;

	MaxValue=65536;
	data.resize(dimX,dimY);
	for (j=0;j<dimY;j++)
		for(i=0;i<dimX;i++){
			h=getc(fd);
			l=getc(fd);
			data(i,j)=l+h*256;
	}
}

void rmImage::readImage16LowEndian(FILE *fd)
{
	int i,j;
	byte h,l;

	MaxValue=65536;
	data.resize(dimX,dimY);
	for (j=0;j<dimY;j++)
		for(i=0;i<dimX;i++){
			h=getc(fd);
			l=getc(fd);
			data(i,j)=l*256+h;
	}
}

/**
** Read Image from a file and store in a rmImage object
**	Parameters: filename : filename to load image from
**
** Byte format: - heading (26 bytes)
**			DimY= 16 (high byte) and 17 (low byte)
**		- Image data: reading in rows
**/
void rmImage::readByte(char *filename)
{
	char *head;
	FILE * fd;
	int aux;

	fd=fopen(filename,"r");

	if (fd==NULL){
		//cout <<"Error Opening Image"<<endl;
		error("Error Opening Image");
	}
	else {
		head=new char[26];
		if (fread(head,1,26,fd)!=26)
			cout <<"Error Reading Image heading";
		else
		{
			// Read image dimensions
			aux = head[16];
			dimY= aux << 8;
			aux = head[17] & 0xff;
			dimY+= aux+1;
			aux = head[24];
			dimX= aux << 8;
			aux = head[25] & 0xff;
			dimX+= aux+1;
			readImage8R(fd);


















		}
		delete head;
		fclose(fd);
	}
}

/**
llegeix una imatge en format tiff
**/
void rmImage::readTiff(char *filename)
{
  FILE * fd, *fd1;
  int l;
  char *cmd;

  fd=fopen(filename,"r");

  cout << "TIFF image: "<<filename<< endl;
	if (fd==NULL)	{
	  error("Opening TIFF Image");
  }
  cmd = new char[30+strlen(filename)];
  strcpy(cmd,"Tiff/obreTiff ");
  for(l=0;l<strlen(filename);l++) cmd[l+14]=filename[l];
  cmd[14+strlen(filename)]='\0';
  cout<< cmd <<endl;
//  l=system("/home/Joan/Tiff/obreTiff /home/Joan/Tiff/101r.tif");
  l=system(cmd);
  cout << l;
//  dimX=100; dimY=100;
  fd1=fopen("Tiff/prova.dat","r");
  fread(&dimX,sizeof(uint32),1,fd1);
  fread(&dimY,sizeof(uint32),1,fd1);
  MaxValue=256;

  readImage8(fd1);
  fclose(fd1);
//  readImage8("/home/Joan/Tiff/prova.dat");
/*	else {
    TIFFGetField(inimg, TIFFTAG_IMAGEWIDTH, &width);
    TIFFGetField(inimg, TIFFTAG_IMAGELENGTH, &height);
    TIFFGetField(inimg, TIFFTAG_BITSPERSAMPLE, &bitspersample);
   }
 */
	fclose(fd);
}

void rmImage::readPgm(char *filename){
FILE *fd;
  char c;
  int amplada,alcada;
    fd=fopen(filename,"r");
		while(c!=10){ // lecura de la identificaci del fitxer
			fread(&c,sizeof(char),1,fd);
		}
		amplada=0;
		while(c!=32){ // lectura de l'amplada
			fread(&c,sizeof(char),1,fd);
			if(c!=32){
			amplada=amplada+c-48;
			amplada=amplada*10;
			}
		}
		amplada=amplada/10;
		alcada=0;
		while(c!=10){ // lectura de l'alcada
			fread(&c,sizeof(char),1,fd);
			if(c!=10){
				alcada=alcada+c-48;
				alcada=alcada*10;
			}
		}
		alcada=alcada/10;
    dimX = amplada;
    dimY = alcada;
    c=0;
    while(c!=10){ // lectura de la profunditat
			fread(&c,sizeof(char),1,fd);
    }
    //cout << "Amplada: " << amplada << " Alcada: " << alcada <<endl;
    readImage8(fd);
    fclose(fd);  }

void rmImage::readRaw(char *filename)
{
  FILE * fd;

  fd=fopen(filename,"r");
//  dimX = 1524;
//  dimY = 1012;

  cout << "RAW image: "<<filename<< endl;
	if (fd==NULL)	{
	  error("Opening RAW Image");
  }
	else {
    while(rbcaps>0){
      fgetc(fd);
      rbcaps--;
    }
    if(rfrmt==0) readImage16LowEndian(fd);
    else readImage16(fd);
  }

	fclose(fd);

}

/**
** Save Image
**	parameters : -filename to save
**/
void rmImage::save(char *filename)
{
	format=findFormat(filename);
	switch (format)
	{
		case RM_MIT:saveMit(filename);break;
		case RM_BYTE:saveByte(filename);break;
    case RM_RAW:saveRaw(filename);break;

		case RM_NO_FORMAT:error("Unknown file format");break;
	}
}

/**
** Save MIT Image
**    parameters : -filename to save
**/
void rmImage::saveMit(char *filename)
{
	FILE * fd;

	fd=fopen(filename,"w");
	cout << "MIT image: "<<filename<< endl;
	if (fd==NULL) {
		error("Creating MIT Image");
	}
	else
	{
		saveMitHeader(fd);
		switch (bits)
		{
			case 8: saveImage8(fd);break;
			case 12:saveImage12(fd);break;
			case 16:saveImage16(fd);break;
		}
	}
	fclose(fd);

}


/**

** Save MIT Image
**    parameters : -filename to save
**/
void rmImage::saveRaw(char *filename)
{
	FILE * fd;

	fd=fopen(filename,"w");
	cout << "RAW image: "<<filename<< endl;
	if (fd==NULL) {
		error("Creating RAW Image");
	}
	else{
    saveImage16LowEndian(fd);
	}
	fclose(fd);
}

void rmImage::saveMitHeader(FILE *fp)
{
  int low,high;

  ftype=1;	// Mit header format
  high = ftype / 256;
  low  = ftype % 256;
  putc(low,fp);
  putc(high,fp);

  high = bits  / 256;
  low  = bits  % 256;

  putc(low,fp);
  putc(high,fp);

  high = dimX / 256;
  low  = dimX % 256;
  putc(low,fp);
  putc(high,fp);

  high = dimY / 256;
  low  = dimY % 256;
  putc(low,fp);

  putc(high,fp);
}


void rmImage::saveImage8(FILE *fd)
{
	long i,j;
	for (j=0;j<dimY;j++)
		for(i=0;i<dimX;i++) {
			putc(char(data(i,j)),fd);
	}
}

void rmImage::saveImage12(FILE *fd)
{
	saveImage16(fd);
}

void rmImage::saveImage16(FILE *fd)
{
	int high, low,i,j;
	for (j=0;j<dimY;j++)

		for(i=0;i<dimX;i++){

			high = data(i,j)/ 256;
			low  = data(i,j) % 256;
			putc(high,fd);
			putc(low,fd);
	}
}

void rmImage::saveImage16LowEndian(FILE *fd)

{
	int high, low,i,j;
	for (j=0;j<dimY;j++)
		for(i=0;i<dimX;i++){
			low = data(i,j)/ 256;
			high  = data(i,j) % 256;
			putc(high,fd);
			putc(low,fd);
	}
}

/**
** Save Image into a file from a rmimage object
**      Parameters: filename : filename to save
**
** File format: - heading (26 bytes)

** 								DimY= 16 (high byte) and 17 (low byte)
**              - Image data
**							- footer: 64 bytes
**/
void rmImage::saveByte(char *filename)
{
	char *head;
	FILE * fd;
	int aux,i,j;
	long n;
	byte * buffer;

	fd=fopen(filename,"w");

	if (fd==NULL)	{
		error ("saveByte: creating file");
	}
	else{
		head=new char[26];
		for (i=0;i<26;i++) head[i]=0;
		head[1]=head[3]=head[7]=head[9]=1;
		aux=dimY-1;
		head[17]=char(aux%256 );
		head[16]=char(aux/256);
		aux=dimX-1;
		head[25]=char((aux)%256);

		head[24]=char(aux/256);

		if (fwrite(head,1,26,fd)!=26) {
			error("Writing Image heading");
		}

		else {
			buffer= new byte [dimX*dimY];
			n=0;
			for (j=0;j<dimY;j++)

				for(i=0;i<dimX;i++) {
					buffer[n]=data(i,dimY-1-j);
					n++;
			}
			if ((int)fwrite(buffer,1,dimX*dimY,fd)!=dimX*dimY) {
      	error("Writing Image data");
			}
			delete head;
			head= new char [74];
			for (i=0;i<74;i++) head[i]=0;
			sprintf(head,"RD_PsWindow");
			head[13]=head[15]=head[17]=head[19]=head[21]=head[49]=1;
			head[30]=head[31]=head[32]=head[33]=head[42]=
			head[43]=head[44]=head[45]=0xff;
			head[47]=head[60]=head[64]=head[72]=2;
			head[73]=0xc3; head[61]=0xc4; head[65]=0x12;

			if (fwrite(head,1,64,fd)!=64) {
				error("Writing footer data");
			}
			delete head;
			delete buffer;
		}
		fclose(fd);
	}
}

rmImage::~rmImage()
{
}

/**
** Transform an image by a teta rotation and a ux,uy translation
**/
void rmImage::transformRigid(double teta, double ux,double uy)
{

	int j,i,k,l;
	rmImage bak;
	rmMatrix2d<float> m(3,3),n(3,1);

	bak.resize(dimX,dimY);
	bak.data.clear(0);

	m(1,1)=cos(teta); m(1,2)=sin(teta); m(1,3)= ux;
	m(2,1)=-sin(teta); m(2,2)=cos(teta); m(2,3)=uy;
	m(3,1)=0;m(3,2)=0;m(3,3)=1;

	m=m.inverse();
	for (j=0;j<dimY;j++)
  	for(i=0;i<dimX;i++) {
			n(1,1)=i; n(2,1)=j;n(3,1)=1;
			n=m*n;
			k=(int)rint(n(1,1));
			l=(int)rint(n(2,1));
			if ((k>=0)&&(k<dimX)&&(l>=0)&&(l<dimY))
				bak.data(i,j)=data(k,l);
	}
	data=bak.data;
}

/**

** Transform an image by a teta rotation and a ux,uy translation
** 			and scaling scale
**/
void rmImage::transformScale(double ux,double uy,double teta, double scale )
{
	int j,i,k,l;
	rmImage bak;
	rmMatrix2d<float> m(3,3),n(3,1);

	bak.resize(dimX,dimY);
	bak.data.clear(128);

	m(1,1)=scale*cos(teta); m(1,2)=scale*sin(teta);  m(1,3)= ux;
	m(2,1)=-scale*sin(teta); m(2,2)=scale*cos(teta); m(2,3)=uy;
	m(3,1)=0;m(3,2)=0;m(3,3)=scale*1;
	m=m.inverse();
	for (j=0;j<dimY;j++)
		for(i=0;i<dimX;i++){
			n(1,1)=i; n(2,1)=j;n(3,1)=1;
			n=m*n;
			k=int(n(1,1));l=int(n(2,1));
			if ((k>=0)&&(k<dimX)&&(l>=0)&&(l<dimY))
				bak.data(i,j)=data(k,l);
	}
	data=bak.data;
}



/**
** Negate image greylevels
**/
void rmImage::negate()
{
	int j,i;

	for (j=0;j<dimY;j++)
		for(i=0;i<dimX;i++) {
			data(i,j) =(MaxValue-1)-data(i,j);
	}
}

/**
** Translate an image ux,uy
**/
void rmImage::translate(double ux,double uy)
{
	int j,i,k,l;
	rmImage bak;



	bak.resize(dimX,dimY);
	for (j=0;j<dimY;j++)
		for(i=0;i<dimX;i++) {
			k=int(i+ux);l=int(j+uy);
			if ((k>=0)&&(k<dimX)&&(l>=0)&&(l<dimY))
				bak.data(i,j)=data(k,l);
	}
	data=bak.data;
}


/**
** Cut image taking x0,y0,x1,y1
**/
void rmImage::cut(int x0,int y0,int x1,int y1)
{
	int j,i;
	rmImage bak;

	if (outLimit(x0,y0,0) || outLimit(x1,y1,0)) {
		cout << "Can't Cut" <<endl;
	}
	else {
		bak.resize(x1-x0+1,y1-y0+1);

		for (j=y0;j<=y1;j++)
			for(i=x0;i<=x1;i++) {
				bak.data(i-x0,j-y0)=data(i,j);
		}
		resize(x1-x0+1,y1-y0+1);
    data=bak.data;
	}
}

/**
** Enlarge image taking offset x, ofy ,dimX,dimY
**/
void rmImage::enlarge(int ofx,int ofy,int dx,int dy)
{
	int j,i;
	rmImage bak;

	bak.resize(dx,dy);
	bak.data.clear(0);


	if (!bak.outLimit(dimX-1,dimY-1,0)) {
		for (j=0;j<dimY;j++) {
			for(i=0;i<dimX;i++) {
				bak.data(i+ofx,j+ofy)=data(i,j);
      }
		}
		resize(dx,dy);
    data=bak.data;
	} else cout <<"Enlarge: new image is smaller than original"<< endl;
}



/**
** Subtract images this = this - s* ima
**/
void rmImage::subtract(float s,const rmImage &ima)

{
	int j,i,dX,dY;
	dX=MinVal(dimX,ima.dimX);
	dY=MinVal(dimY,ima.dimY);

	// Should match image dimensions.DO!
	if ((s<0)||(s>1)) error ("scaling factor must be 0..1 in Subs(image)");
 	for (j=0;j<dY;j++)
  	for(i=0;i<dX;i++) {
			data(i,j)= abs(int(data(i,j)-(s*ima.data(i,j))));
		}
}

/**
** Obtain Maximum image   this = max(this,ima)
**/
void rmImage::operationMax(rmImage ima)
{
	int j,i;
	// Should match image dimensions. todo!
	for (j=0;j<dimY;j++)
		for(i=0;i<dimX;i++) {
			if (data(i,j) < ima.data(i,j)) data(i,j) = ima.data(i,j);
		}
}

/**
** Obtain And image   this = this & ima)
**/
void rmImage::operationAnd(rmImage ima)
{
	int j,i;
	// Should match image dimensions. todo!
	for (j=0;j<dimY;j++)
		for(i=0;i<dimX;i++)
		{
			data(i,j) = data(i,j) & ima.data(i,j);
		}
}

/**
** Obtain bounding box coordinates (0..p.x) (0..p.y)
**/

rmPoint2d rmImage::getBBox()

{
	rmPoint2d p;
	int j,i;
	p.x(0);p.y(0);
	for (j=0;j<dimY;j++)
		for(i=0;i<dimX;i++) {
			if (data(i,j)>0) {
				if (i>p.x()) p.x(i);
				if (j>p.y()) p.y(j);
			}
	}


	return p;
}


/** returns p[0] low bound and p[1] high bound
**/
void rmImage::getBBox(rmPoint2d p[2])
{
	int j,i;
	p[0].x(dimX);p[0].y(dimY);
	p[1].x(0);p[1].y(0);
	for (j=0;j<dimY;j++)
		for(i=0;i<dimX;i++){
			if (data(i,j)>0) {
		  	if (i<p[0].x()) p[0].x(i);
				if (j<p[0].y()) p[0].y(j);
				if (i>p[1].x()) p[1].x(i);
				if (j>p[1].y()) p[1].y(j);
			}
	}
}



/**
** Mirror an image by X coordinate

**/
void rmImage::mirrorX()
{
	int j,i;
	rmImage bak;

	bak.resize(dimX,dimY);

	for (j=0;j<dimY;j++)
		for(i=0;i<dimX;i++) {
			bak.data(dimX-1-i,j)= data(i,j);
	}
	data=bak.data;
}

void rmImage::mirrorY()
{
	int j,i;
	rmImage bak;

	bak.resize(dimX,dimY);
	for (j=0;j<dimY;j++)
		for(i=0;i<dimX;i++) {
			bak.data(i,dimY-1-j)= data(i,j);
	}
	data=bak.data;
}






void rmImage::normDICOM()
{

	rmHistogram h(MaxValue);
	int range[2];

	h.create(*this);
	h.readRange(range);
	if ((bits>8)&&(range[1]<=256))
	{
	   cout << "not using all bits: down to 8"<< endl;
	   bits =8;
	   MaxValue = 256;
	}else cout << "bits " << bits << endl;
}

void rmImage::norm()
{
	rmHistogram h(MaxValue);
	double ratio;
	int range[2],i,j;

	h.create(*this);
	h.readRange(range);
	ratio= (MaxValue-1)/double(range[1]-range[0]);

	for (j=0;j<dimY;j++)

		for(i=0;i<dimX;i++) {
			data(i,j)=imdata((data(i,j)-range[0])*ratio);
	}
}

void rmImage::resize(int x, int y)
{
	dimX=x;

	dimY=y;
	data.resize(x,y);
}

void rmImage::reduce()
{

	/*
	RD_Image<imdata> out(data.xlo(),data.xhi(),data.ylo(),data.yhi());


	SM_GaussReduce<imdata>(out,data);
	data=out;
	dimX=int(data.xlo()+data.xhi()+1);
	dimY=int(data.ylo()+data.yhi()+1);
	cout << "Reduction: new dimensions " << dimX<<" " << dimY <<endl;
	*/
	cout << "todo" << endl;

}

void rmImage::expand()

{
	/*
	RD_Image<imdata> out(data.xlo(),data.xhi(),data.ylo(),data.yhi());

	SM_GaussExpand<imdata>(out,data);
	data=out;
	dimX=int(data.xlo()+data.xhi()+1);
	dimY=int(data.ylo()+data.yhi()+1);
	cout << "Expansion: new dimensions " << dimX<<" " << dimY <<endl;
	*/
	cout << "todo" << endl;
}


/** Reduce Bits
**	Change the pixel depth to bits
**/
void rmImage::reduceBits(int nb)
{
	long no;
	int i,j;

	if (bits > nb)
	{
		no=1<<nb;
		cout << "No. "<< no<<endl;

		for (j=0;j<dimY;j++)
			for(i=0;i<dimX;i++) {

				data(i,j)=imdata(float(data(i,j)*no)/float(MaxValue));
		}

		bits=nb;
		MaxValue=no;
	}
	else cout <<"reduceBits: image can't be reduced"<<endl;

}



/*
////////////////////
// Reduce Bits for the Phantom image (2000-2768)
///////////////////
void rmImage::RBitsPhantom(int nb)
{
    long no;
    int i,j;
    imdata **ptr;
    long PM,Pm;

    if (bits > nb)
    {
	PM = 2768;
	Pm = 2000;
        ptr=data.dataPtr();
        no=(1<<nb)-1;
        cout << "Phantom Reduction No. "<< no<<endl;
        for (j=0;j<dimY;j++)
         for(i=0;i<dimX;i++)
         {
		if (ptr[j][i]>PM) ptr[j][i]=PM;
		if (ptr[j][i]<Pm) ptr[j][i]=Pm;

                ptr[j][i]=long((ptr[j][i]-Pm)*no)/(PM-Pm);
         }
        bits=nb;
   	MaxValue=no;
    }
    else cout <<"RBits: image can't be reduced"<<endl;

}
*/

/**
** Add Gaussian noise to the image
**/
void rmImage::noise(float m,float std)
{
	/*
	imdata **ptr;
	int i,j;
	SM_Random rand;

	int value;

 	ptr=data.dataPtr();
        for (j=0;j<dimY;j++)
         for(i=0;i<dimX;i++)
         {
	    	value = int(rand.gaussian(m,std));
		if ((ptr[j][i]+value)>=MaxValue) ptr[j][i]=MaxValue-1;
		else if ((ptr[j][i]+value)<0)  ptr[j][i]=0;
        	     else ptr[j][i]=ptr[j][i]+value;
         }
	*/
	cout << "todo" << endl;
}


/**
** Check if position x,y is out of the limits of the image plus s pixels
**/
int rmImage::outLimit(int x,int y,int s)
{
        if ((x<s)||(x>=(dimX-s))||(y<s)||(y>=(dimY-s))) return true;
        else return false;
}

/**
** Apply Threshold to an image
**/
void rmImage::threshold(int td,int min, int max)
{
	int i,j;

	for (j=0;j<dimY;j++)
		for(i=0;i<dimX;i++){
			if (data(i,j)<td) data(i,j)=min;
			else data(i,j)=max;
	}
}


/**
** Count the number of pixels >= td in the image
**/
double rmImage::area(int td)
{
	int i,j;
	double num=0;

	for (j=0;j<dimY;j++)
		for(i=0;i<dimX;i++) {
			if (data(i,j)>=td) num++;
	}
	return num;
}


/**
** Get the maximum cluster area, as well as the number of clusters in the image
**/
/*
long rmImage::MaxCluster(int td,int &numCluster, RD_PsIrregular & Maxregion, RD_PsBoundary &bound)
{

   rmImage temp;

   temp.Resize(dimX,dimY);
   temp.data= data;
   temp.Threshold(td,0,MaxValue-1);

   RD_Image<byte> byteimage;
   temp.CopyTo(byteimage);
   RD_BlobOp grabber(byteimage.rvd(),byteimage);
   RD_Blob lump;
   //RD_List<RD_Blob> Llump;

   long size=0,max_size=0;
   RD_PsIrregular region;


   numCluster=0;
   grabber.reset();
   //Llump.makeEmpty();
   while (grabber.nextBlob(lump)) {
        region=lump.region();
        size=region.npoints();
        //cout << "Size of the region" << size << endl;
        if (size>max_size) {
	    max_size=size;
	    Maxregion= region;
	    bound = lump.boundary();
	}
	numCluster++;
        //Llump.append(lump);
   }
   //cout << "Maximum region size" << max_size<< endl;

   //cout << "Total regions " << Llump.nelems()<< endl;
   //numCluster = Llump.nelems();
//cout <<  "Total regions " << numCluster;
//cout << "Maximum region size" << max_size<< endl;
   return max_size;
}
*/

/**
** Return the number of different grey-levels of the image
**/
int rmImage::numGrey()
{
  int i;
	rmHistogram h(MaxValue);
	int num=0;

	h.create(*this);
	for (i=0;i<MaxValue;i++)
		if(h.readValue(i)>0) num++;
	return num;
}

void rmImage::operator =(const rmImage &im)
{

	int i,j;

	resize(im.dimX,im.dimY);
	for (j=0;j<dimY;j++)
		for(i=0;i<dimX;i++)
			data(i,j) = im.data(i,j);
	bits = im.bits;
	format = im.format;
	MaxValue = im.MaxValue;
	for (i=0;i<3;i++) voxelSize[i]= im.voxelSize[i];
	sliceSpacing=im.sliceSpacing;
}



void rmImage::copyFrom(unsigned char pim[1][1])
{
	int i,j;
	for (j=0;j<dimY;j++)
		for(i=0;i<dimX;i++)
			data(i,j) =imdata(pim[i][j]);
}

void rmImage::copyTo(imdata *ptr)
{
	int i,j;
	for (j=0;j<dimY;j++)
		for(i=0;i<dimX;i++)
			ptr[i+dimX*j]=imdata(data(i,dimY-1-j));

}



void rmImage::setBits(int b) {
   bits = b;
}

int rmImage::getBits() {
   return bits;

}

void rmImage::gaussSmooth(float sd)
{
   int i,j;
   rmImage dest;
   rmImage aux;
   dest.resize(dimX,dimY);
   aux.resize(dimX,dimY);

   // Gaussian 1 3 1 in x direction
   for (j=0;j<dimY;j++)
   {
       for (i=1;i<dimX-1;i++) {
          aux.data(i,j) = (imdata) (( data(i-1,j) + data(i+1,j)
                         + 3.0 * data(i,j) ) / 5.0 );
       }
       // Now deal with edge effects : (from Radial library GaussRed!)
       // For 0'th element use ( 3 1 ) :
       aux.data(0,j) = (imdata) (( 3.0 * data(0,j)
                     + data(1,j) ) / 4.0);

       // For last element use ( 1 3 ) :
       aux.data(dimX-1,j) = (imdata) (( 3.0 * data(dimX-1,j)
                        + data(dimX-2,j)) / 4.0);
   }

   // Gaussian in y direction  (1 3 1)
   if (dimY>1) {
     for (i=0;i<dimX;i++)
     {
      for (j=1;j<dimY-1;j++)
      {
      	dest.data(i,j) = (imdata)((aux.data(i,j-1)
					+ aux.data(i,j+1) + 3.0  * aux.data(i,j) ) / 5.0);
			}
      // Now edge effects :
      // Use (3 1) filter for 0th element.
      dest.data(i,0) = (imdata) ((3.0 * aux.data(i,0)
                 + aux.data(i,1)) / 4.0 );

      dest.data(i,dimY-1) = (imdata) (( 3.0 * aux.data(i,dimY -1)
                 + aux.data(i,dimY-2) )/4.0);
     }
     data = dest.data;
   } else data = aux.data;
}

//Afegit per Gerard Alcorlo Bofill ;-)

/* Conversio a 8bpp */
void rmImage::escalaGris(){
  for(int i=0;i<dimX;i++){
   for(int j=0;j<dimY;j++){
       data(i,j) = (uint8) data(i,j);
   }
  }
}

void rmImage::threshold_2(int td,int min, int max, rmImage &rm)
{
	int i,j;

   if (td==min)
      for (j=0;j<dimY;j++)
		for(i=0;i<dimX;i++)
               data(i,j)=max;

  else if (td==max)
      for (j=0;j<dimY;j++)
      	for(i=0;i<dimX;i++)
               data(i,j)=min;
  else{   
      for (j=0;j<dimY;j++)
 		for(i=0;i<dimX;i++){
 			if (rm.data(i,j)<td)
                       data(i,j)=min;
 			else if (rm.data(i,j)>td)
                      data(i,j)=max;
 	}
  }
}


void rmImage::setup_brightness_lut(int thresh)
{
int   k;
float temp;

  bp=new imdata[516];
//  bp=(imdata *)malloc(516);
  bp=bp+258;

  for(k=-256;k<257;k++)
  {
    temp=((float)k)/((float)thresh);
    temp=temp*temp;

    temp=temp*temp*temp;
    temp=100.0*exp(-temp);
    *(bp+k)= (imdata) temp;
  }
}


int rmImage::susan_corners()
{
int   n,x,y,sq,xx,yy, i,j,*cgx,*cgy;
float divide;
imdata c,*p,*cp, *in;
int *r, max_no=1850, x_size, y_size;

   x_size=dimX;
   y_size=dimY;

  in=new imdata[x_size*y_size];
  copyTo (in);

  r = new int[x_size*y_size];    
  for (i=0;i<x_size*y_size;i++)  //inicialitzem l'espai
      r[i]=(imdata)0;


  corner_list = new complexe[15000];   


  cgx=new int[x_size*y_size]; 
  cgy=new int[x_size*y_size]; 

  for (i=5;i<y_size-5;i++)
    for (j=5;j<x_size-5;j++) {
        n=100;
        p=in + (i-3)*x_size + j - 1;   //hi ha un in
        cp=bp + in[i*x_size+j];         //un atre in

        n+=*(cp-*p++);
        n+=*(cp-*p++);
        n+=*(cp-*p);
        p+=x_size-3;

        n+=*(cp-*p++);
        n+=*(cp-*p++);
        n+=*(cp-*p++);
        n+=*(cp-*p++);
        n+=*(cp-*p);
        p+=x_size-5;

        n+=*(cp-*p++);
        n+=*(cp-*p++);
        n+=*(cp-*p++);
        n+=*(cp-*p++);
        n+=*(cp-*p++);
        n+=*(cp-*p++);

        n+=*(cp-*p);
        p+=x_size-6;

        n+=*(cp-*p++);
        n+=*(cp-*p++);
        n+=*(cp-*p);
      if (n<max_no){    /* do this test early and often ONLY to save wasted computation */
        p+=2;
        n+=*(cp-*p++);
      if (n<max_no){
        n+=*(cp-*p++);
      if (n<max_no){

        n+=*(cp-*p);
      if (n<max_no){
        p+=x_size-6;

        n+=*(cp-*p++);
      if (n<max_no){
        n+=*(cp-*p++);
      if (n<max_no){
        n+=*(cp-*p++);
      if (n<max_no){
        n+=*(cp-*p++);
      if (n<max_no){
        n+=*(cp-*p++);
      if (n<max_no){
        n+=*(cp-*p++);
      if (n<max_no){
        n+=*(cp-*p);
      if (n<max_no){
        p+=x_size-5;

        n+=*(cp-*p++);
      if (n<max_no){
        n+=*(cp-*p++);
      if (n<max_no){
        n+=*(cp-*p++);
      if (n<max_no){
        n+=*(cp-*p++);
      if (n<max_no){
        n+=*(cp-*p);
      if (n<max_no){
        p+=x_size-3;

        n+=*(cp-*p++);
      if (n<max_no){
        n+=*(cp-*p++);
      if (n<max_no){
        n+=*(cp-*p);

        if (n<max_no)
        {
            x=0;y=0;
            p=in + (i-3)*x_size + j - 1;               //un atre in


            c=*(cp-*p++);x-=c;y-=3*c;
            c=*(cp-*p++);y-=3*c;
            c=*(cp-*p);x+=c;y-=3*c;
            p+=x_size-3;

            c=*(cp-*p++);x-=2*c;y-=2*c;
            c=*(cp-*p++);x-=c;y-=2*c;
            c=*(cp-*p++);y-=2*c;
            c=*(cp-*p++);x+=c;y-=2*c;
            c=*(cp-*p);x+=2*c;y-=2*c;
            p+=x_size-5;

            c=*(cp-*p++);x-=3*c;y-=c;
            c=*(cp-*p++);x-=2*c;y-=c;
            c=*(cp-*p++);x-=c;y-=c;

            c=*(cp-*p++);y-=c;
            c=*(cp-*p++);x+=c;y-=c;
            c=*(cp-*p++);x+=2*c;y-=c;
            c=*(cp-*p);x+=3*c;y-=c;
            p+=x_size-6;

            c=*(cp-*p++);x-=3*c;
            c=*(cp-*p++);x-=2*c;
            c=*(cp-*p);x-=c;
            p+=2;
            c=*(cp-*p++);x+=c;
            c=*(cp-*p++);x+=2*c;
            c=*(cp-*p);x+=3*c;
            p+=x_size-6;

            c=*(cp-*p++);x-=3*c;y+=c;
            c=*(cp-*p++);x-=2*c;y+=c;
            c=*(cp-*p++);x-=c;y+=c;
            c=*(cp-*p++);y+=c;
            c=*(cp-*p++);x+=c;y+=c;
            c=*(cp-*p++);x+=2*c;y+=c;
            c=*(cp-*p);x+=3*c;y+=c;

            p+=x_size-5;

            c=*(cp-*p++);x-=2*c;y+=2*c;
            c=*(cp-*p++);x-=c;y+=2*c;
            c=*(cp-*p++);y+=2*c;
            c=*(cp-*p++);x+=c;y+=2*c;

            c=*(cp-*p);x+=2*c;y+=2*c;
            p+=x_size-3;

            c=*(cp-*p++);x-=c;y+=3*c;
            c=*(cp-*p++);y+=3*c;
            c=*(cp-*p);x+=c;y+=3*c;

            xx=x*x;
            yy=y*y;
            sq=xx+yy;
            if ( sq > ((n*n)/2) )
            {
              if(yy<xx) {
                divide=(float)y/(float)abs(x);
                sq=abs(x)/x;
                sq=*(cp-in[(i+FTOI(divide))*x_size+j+sq]) +              //3 ins!!!!!!
                   *(cp-in[(i+FTOI(2*divide))*x_size+j+2*sq]) +
                   *(cp-in[(i+FTOI(3*divide))*x_size+j+3*sq]);}
              else {
                divide=(float)x/(float)abs(y);
                sq=abs(y)/y;
                sq=*(cp-in[(i+sq)*x_size+j+FTOI(divide)]) +           //3 ins!!!!!
                   *(cp-in[(i+2*sq)*x_size+j+FTOI(2*divide)]) +
                   *(cp-in[(i+3*sq)*x_size+j+FTOI(3*divide)]);}

              if(sq>290){
                r[i*x_size+j] = max_no-n;
                cgx[i*x_size+j] = (51*x)/n;
                cgy[i*x_size+j] = (51*y)/n;}
            }
	}
}}}}}}}}}}}}}}}}}}}

  /* to locate the local maxima */
  n=0;
  for (i=5;i<y_size-5;i++)
    for (j=5;j<x_size-5;j++) {
       x = r[i*x_size+j];
       if (x>0)  {
          /* 5x5 mask */
#ifdef FIVE_SUPP
          if (

              (x>r[(i-1)*x_size+j+2]) &&
              (x>r[(i  )*x_size+j+1]) &&
              (x>r[(i  )*x_size+j+2]) &&
              (x>r[(i+1)*x_size+j-1]) &&
              (x>r[(i+1)*x_size+j  ]) &&
              (x>r[(i+1)*x_size+j+1]) &&
              (x>r[(i+1)*x_size+j+2]) &&
              (x>r[(i+2)*x_size+j-2]) &&
              (x>r[(i+2)*x_size+j-1]) &&
              (x>r[(i+2)*x_size+j  ]) &&
              (x>r[(i+2)*x_size+j+1]) &&
              (x>r[(i+2)*x_size+j+2]) &&

              (x>=r[(i-2)*x_size+j-2]) &&
              (x>=r[(i-2)*x_size+j-1]) &&
              (x>=r[(i-2)*x_size+j  ]) &&
              (x>=r[(i-2)*x_size+j+1]) &&
              (x>=r[(i-2)*x_size+j+2]) &&
              (x>=r[(i-1)*x_size+j-2]) &&
              (x>=r[(i-1)*x_size+j-1]) &&
	      (x>=r[(i-1)*x_size+j  ]) &&
	      (x>=r[(i-1)*x_size+j+1]) &&
	      (x>=r[(i  )*x_size+j-2]) &&
	      (x>=r[(i  )*x_size+j-1]) &&
	      (x>=r[(i+1)*x_size+j-2]) )
#endif
#ifdef SEVEN_SUPP
          if (
                (x>r[(i-3)*x_size+j-3]) &&
                (x>r[(i-3)*x_size+j-2]) &&
                (x>r[(i-3)*x_size+j-1]) &&
                (x>r[(i-3)*x_size+j  ]) &&
                (x>r[(i-3)*x_size+j+1]) &&
                (x>r[(i-3)*x_size+j+2]) &&
                (x>r[(i-3)*x_size+j+3]) &&

                (x>r[(i-2)*x_size+j-3]) &&
                (x>r[(i-2)*x_size+j-2]) &&
                (x>r[(i-2)*x_size+j-1]) &&
                (x>r[(i-2)*x_size+j  ]) &&
                (x>r[(i-2)*x_size+j+1]) &&
                (x>r[(i-2)*x_size+j+2]) &&
                (x>r[(i-2)*x_size+j+3]) &&

                (x>r[(i-1)*x_size+j-3]) &&
                (x>r[(i-1)*x_size+j-2]) &&
                (x>r[(i-1)*x_size+j-1]) &&
                (x>r[(i-1)*x_size+j  ]) &&
                (x>r[(i-1)*x_size+j+1]) &&
                (x>r[(i-1)*x_size+j+2]) &&
                (x>r[(i-1)*x_size+j+3]) &&

                (x>r[(i)*x_size+j-3]) &&
                (x>r[(i)*x_size+j-2]) &&
                (x>r[(i)*x_size+j-1]) &&
                (x>=r[(i)*x_size+j+1]) &&
                (x>=r[(i)*x_size+j+2]) &&
                (x>=r[(i)*x_size+j+3]) &&

                (x>=r[(i+1)*x_size+j-3]) &&
                (x>=r[(i+1)*x_size+j-2]) &&
                (x>=r[(i+1)*x_size+j-1]) &&
                (x>=r[(i+1)*x_size+j  ]) &&
                (x>=r[(i+1)*x_size+j+1]) &&
                (x>=r[(i+1)*x_size+j+2]) &&
                (x>=r[(i+1)*x_size+j+3]) &&

                (x>=r[(i+2)*x_size+j-3]) &&

                (x>=r[(i+2)*x_size+j-2]) &&
                (x>=r[(i+2)*x_size+j-1]) &&
                (x>=r[(i+2)*x_size+j  ]) &&
                (x>=r[(i+2)*x_size+j+1]) &&
                (x>=r[(i+2)*x_size+j+2]) &&
                (x>=r[(i+2)*x_size+j+3]) &&

                (x>=r[(i+3)*x_size+j-3]) &&
                (x>=r[(i+3)*x_size+j-2]) &&
                (x>=r[(i+3)*x_size+j-1]) &&
                (x>=r[(i+3)*x_size+j  ]) &&
                (x>=r[(i+3)*x_size+j+1]) &&
                (x>=r[(i+3)*x_size+j+2]) &&
                (x>=r[(i+3)*x_size+j+3]) )
#endif
{

corner_list[n].x=j;
corner_list[n].y=y_size-i;

n++;
}}}


delete (cgx);
delete (cgy);
delete (r);
delete (in);

return (n);
}

                                 
bool rmImage::mesD2colors(){
   short a, b, c, val;
   bool final=false, b_trobat=false;
   int i, j;
    /*
  FILE *f;
  f = fopen("/home/galco/colors.txt", "wr");
  
  a=data(0,0);

  for(int j=0;j<dimY && !final;j++){
     for(int i=0;i<dimX && !final;i++){
        fprintf (f, "%9d       ", data(i,j));   //cout << data(i,j) << "  ";
      }
      fprintf (f, "\n");
  }



      fclose (f); 
  */
  
  a=data(0,0);
  for(j=0;j<dimY && !final;j++){
   for(i=0;i<dimX && !final;i++){
      val=data(i,j);
      if (data(i,j)!=a && !b_trobat){
         b=data(i,j);
         b_trobat=true;
      }
      if (a!=data(i,j) && b_trobat)
         if (data(i,j)!=b)
            final=true;
   }
  }      

  return (final);
  
}

QImage rmImage::ompleQImage(){

  QImage qImatge;
  qImatge.create ( dimX,dimY,8,256);
  short valor;


  for (int j=0;j<dimY;j++){
    for(int i=0;i<dimX;i++) {
      valor = data(i,j);
      /*if(valor>255) valor = 255;
      if(valor<0) valor = 0;
      */
      if(format == RM_RAW) valor = (byte) ((double)valor * (255.0 / 65535.0));
      else{
        if(valor>255) valor = 255;
        else if(valor<0) valor = 0;
      }
      qImatge.setPixel(i,j,valor);
    }
  }

  for(int c=0;c<256;c++)
     qImatge.setColor( c,qRgb(c,c,c));

  // canvia la imatge del widget que mostra i el seu tamany

//  dw->canviaImatge(qImatge);
//  dw->setGeometry(0,0,qImatge.width(),qImatge.height());

return qImatge;
}   
   