/***************************************************************************
                          rmimage.h  -  description
                             -------------------
    begin                : Wed Jun 12 2002
    copyright            : (C) 2002 by Robert Marti
    email                : robert@localhost.localdomain
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef RMIMAGE_H
#define RMIMAGE_H

//#include "Rad/Image.h"
//#include "Rad/BlobOp.h"
#include "caract/rmlib/rmimage/rmdicom.h"
#include "caract/rmlib/rmmatrix/rmmatrix2d.h"
#include "caract/rmlib/rmutils/rmpoint2d.h"
#include <tiffio.h>
#include <qimage.h>

#define RM_NO_FORMAT 0
#define RM_MIT 	1
#define RM_BYTE 2
#define RM_DICOM 3
#define RM_TIFF 4
#define RM_RAW 5
#define RM_PGM 6

typedef short imdata;
typedef unsigned char byte;


typedef struct{
                           double x;
                           double y;
                           }complexe;

/**
	2D Image class
  *@author Robert Marti
  */
class rmImage{
public:
	rmMatrix2d<imdata> data;
	int dimX,dimY,bits,format,ftype,MaxValue;
	double voxelSize[3], sliceSpacing;
	rmDICOM dicom;

  // paramatres raw (nomes serveixen pel dialeg)
  int rbcaps,rfrmt; // rbcaps: bits de capcalera. rfrmt: format 0 Little Endian !=0 Big

public:
	rmImage();
	rmImage(char *filename);
	~rmImage();

/**
* Print image info
**/
  void printInfo();

/**
* Read Image from a file and store in rmImage object
*      Parameters: filename : filename to load image from
*
* 	This function determines which format (mit or byte) and
* 	reads the image.
**/
	void read(char *filename);

/**
** Save Image
**	parameters :

	-filename to save
**/
	void save(char *filename);

/**

** Transform image using rigid model
**	parameters :
		- Angle teta
		- translation in x and y
**/
	void transformRigid(double teta, double ux, double uy);

/**
** Transform image using rigid plus scaling model
**	parameters :
		- Angle teta
		- translation in x and y
		- scale value
**/
	void transformScale(double ux, double uy,double teta,double scale);

/**
** Transform image using translation only
**	parameters :
		- translation in x and y
	**/
	void translate(double ux, double uy);

	/** Mirror image in X */
	void mirrorX();
	/** Mirror image in Y */
	void mirrorY();
	/** Normalise image grey-level*/
	void norm();
	/** Normalise image using known DICOM grey level distribution */
	void normDICOM();
	/** Resize image in dimX, dimY*/
	void resize(int,int);
	/** Reduce image using Gaussian reduction*/
	void reduce();
	/** Expand image using Gaussian expansion*/
	void expand();
	/** Reduce grey-level depth to n bits*/
	void reduceBits(int);
  /** Adds gaussian noise*/
	void noise(float mean,float sd);
  /** Cuts the image*/
	void cut(int,int,int,int);
	/** Enlarge the image*/
	void enlarge(int,int,int,int);
	/** this image = this -s*a */
	void subtract(float s,const rmImage &a );
	/** this = max(this,a) */
	void operationMax(rmImage a);
	/** this = this & a */
	void operationAnd(rmImage a);
	/** Get Bounding box 0..bbx */
	rmPoint2d getBBox();
	/** Get Bounding box 0..bbx */
	void getBBox(rmPoint2d p[2]);
	/** Check if x,y position is out of the image (plus and offset o)*/
	int outLimit(int x,int y,int o);
	/** Apply a threshold (td) to an image. If pixel< td then pixel=min else pixel=max*/
	void threshold(int td,int min,int max);
	/** Count the number of pixels > td*/
	double area(int td);
//	long maxCluster(int td,int &numCluster,RD_PsIrregular &, RD_PsBoundary &);
	/** Return the number of grey levels*/
	int numGrey();
	/** copy image from unsigned array into object*/
	void copyFrom(unsigned char [1][1]);
	/** copy image to imdata array from object*/
	void copyTo(imdata *ptr);
	void gaussSmooth(float sd);		// Gaussian smoothing of size sd
	void negate();
	void setBits(int);
	int getBits();
//	void reduceBitsPhantom(int); 	// Reduce bits for phantom ROI

/**
** Read byte image from a file and store in a rmImage object
**	Parameters: filename : filename to load image from
**
** Byte format: - heading (26 bytes)
**			DimY= 16 (high byte) and 17 (low byte)
**		- Image data: reading in rows
**/
	void readByte(char *);

/** llegeix una imatge en format tiff*/
  void readTiff(char *);

/**
* Read a mit image from a file and store in a rmImage object
*      Parameters: filename : filename to load image from MIT format
**/
	void readMit(char *);

/**
* Read a DICOM format Image from a file
*      Parameters: filename : filename to load image from
**/
	void readDICOM (char *);
  void readPgm(char *filename);
  void readRaw(char *filename);
  void saveRaw(char *filename);
	/** Copy operator */

	void operator =(const rmImage &);

  	int findFormat(char *);
  
private:

/**
* Find format of the image (.mit or .byte)
*      Parameters: filename : filename to load image from
*

*      This function determines which format (mit or byte) and
*      reads the image.
*/

	void saveByte(char *);
	void saveMit(char *);

/**
* Read 8 bit Image (Y Reversed)
*      Parameters: fd : file descriptor
*/
	void readImage8R(FILE *);

/**
* Read 8 bit Image
*      Parameters: fd : file descriptor
*/
	void readImage8(FILE *);

/**
** Read 12 bit Image
**      Parameters: fd : filename descriptor
**
**/
	void readImage12(FILE *);

/**
** Read 16 bit Image
**      Parameters: fd : filename descriptor
**
**/
	void readImage16(FILE *);
  void readImage16LowEndian(FILE *);

	void saveImage8R(FILE*);
	void saveImage8(FILE*);
	void saveImage12(FILE *);
	void saveImage16(FILE *);
  void saveImage16LowEndian(FILE *);

/**
* Read MIT header
*      Parameters: fd : filename descriptor
*
*/
	void readMitHeader(FILE *);

	void saveMitHeader(FILE *);

   public:
   void escalaGris();

   void threshold_2(int td,int min, int max, rmImage &rm);
   int susan_corners();
   void setup_brightness_lut(int thresh);
   bool mesD2colors();
   QImage ompleQImage();
   complexe *corner_list;

   private:
   imdata *bp;
   
};


#endif
