#ifndef __IMAGE3D
#define __IMAGE3D

#include "rmimage.h"
#include "caract/rmlib/rmmatrix/rmmatrix3d.h"		// my template matrix 3d
#include "caract/rmlib/rmutils/rmpoint3d.h"

#define RM_NO_FORMAT 0
#define RM_MIT 	1
#define RM_BYTE 2
#define RM_DEN 	3

#ifndef MIN
#define MIN(a, b) ((a) > (b) ? (b) : (a))
#endif

/* constants for .den magic number (map_version field) */
//#define MAP_CUR_VERSION         1       /* current version */
//#define MAP_CUR_VERSION_SWAB    0x0100  /* byte-swapped current version */

//#define MAX_READ_SIZE   8192    /* maximum # of bytes per read(2) call */

typedef unsigned short imdata3d;
//typedef unsigned char imdata3d;

/**
	3D Image class
  *@author Robert Marti
  */
class rmImage3d{
public:
	rmMatrix3d<imdata3d> data;
public:
	
	int dimX,dimY,dimZ,bits,format,ftype,MaxValue;
	double vsX,vsY,vsZ;
	double sliceSpacing;
	float origin[3]; // origin of data

public:
	rmImage3d();
	rmImage3d(int,int,int,int,int,int);//Create  image of dimensions x,y,z
	rmImage3d(int,int,int);//Create  image of dimensions x,y,z
	rmImage3d(char *filename);
	~rmImage3d();
	imdata3d & rmImage3d::operator() (int x, int y, int z);
	const imdata3d & rmImage3d::operator() (int x, int y, int z) const;
	void operator=(const rmImage3d &);
	void resize(int,int,int,int,int,int);
	void resize(int,int,int);
	void resize(int *);
	void setOrigin(float *);
	void readImage(char **,int,int,int,char);//Read volume from a set of slices.
	void readImage(char *filename);

	void saveImage(char *filename);
	void getSlice(int dim, int num, rmImage &slice);
	void setSlice(int dim, int num, rmImage &slice);
	void copyTo(imdata3d *ptr);	// used in vtk format [z][y][x]!
	void copyFrom(imdata3d *ptr);
//	void buildVolume(rmImage &);
	void rotateX(double);
//	void rotateY(double);
//	void rotateZ(double);
//	void translate(double,double,double);
	void copyFrom (rmImage3d r);
	template<class T> void copyFrom (const rmMatrix3d<T> &);
	template<class T> void copyTo (rmMatrix3d<T> &);
	void normalise();
	void threshold(int,int,int);
	void reduceBits();		// reduces image to 8 bits 
	void gaussSmooth();		// Reduce using gaussian kernel
	void gaussReduce();		// Reduce using gaussian kernel
	
	void printInfo();	 // print image information.
	void getBBox(rmPoint3d p[2]);
	void enlarge(int, int, int, int, int, int);
 	void allocateImage();
	void deleteImage();
	void clear(int value);
	int outLimit(int, int, int, int);
	void cut(rmPoint3d, rmPoint3d);
	void mirrorX();

	void mirrorY();
	void operationAnd(rmImage3d im);
	void operationSub(rmImage3d im);	// Subtract from another image.
	void invert(); // 255-image
	
private:
	int findFormat(char *);
	void readDen(char *);

	/*int read_bytes(int fd, char *buf, int bytecount);
	int read_shorts(int fd, short *sbuf, int shortcount, int swap);
	int read_words(int fd, int *wbuf, int wordcount, int swap);
	int write_bytes(int fd, char *buf, int bytecount);*/
};
#endif 
