/***************************************************************************
                          rmmatrix2d.h  -  description
                             -------------------
    begin                : Wed Jun 12 2002
    copyright            : (C) 2002 by Robert Marti
    email                : robert@localhost.localdomain
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef RMMATRIX2D_H
#define RMMATRIX2D_H

#include "caract/rmlib/rmmatrix/rmvector.h"
#include <iostream.h>

/**
	*Generic 2D Matrix class
  *@author Robert Marti
  */

template<class T>
class rmMatrix2d{
private: 
	T *data;
public:
	int dimX,dimY;
	int orgX, orgY;

public:
	rmMatrix2d();
  /** Create matrix starting at (0,0) and ending at (nx-1,ny-1)
	*/
	rmMatrix2d(int nx,int ny);
  /** Create matrix starting at (ox,oy) and ending at (nx-1,ny-1)
	*/
	rmMatrix2d(int ox,int oy,int nx, int ny);
	/** Copy constructor*/
	rmMatrix2d(rmMatrix2d<T> &im1);
	~rmMatrix2d();
	T & operator() (int x, int y);
	const T & operator() (int x, int y) const;
	void operator=(const rmMatrix2d<T> &);
	void resize(int,int);
	void resize(int,int,int,int);
	void resize(int *);
	/** Clears matrix using value T*/
	void clear (T);
	/** Prints (cout) matrix parameters */
	void printInfo();
	/** Returns the determinant*/
	double determinant();
	/** Returns the inverse matrix */
	rmMatrix2d<T> inverse();
	/** Returns the transpose matrix*/
	rmMatrix2d<T> transpose();
	/** Returns a vector from the row r */
 	rmVector<T> getRow(int r);
	/** Returns a vector from the column c */
	rmVector<T> getCol(int c);
	/** Set column c with contents of vector v */
	void setCol(int c,const rmVector<T> &v);
	/** Set row r with contents of vector v */
	void setRow(int r,const rmVector<T> &v);	

/** Matrix multiplication mxn */
	friend rmMatrix2d<T> operator* <>(const rmMatrix2d<T> & m,const rmMatrix2d<T> &n);
/** Matrix sum m+n */
	friend rmMatrix2d<T> operator+ <>(const rmMatrix2d<T> &,const rmMatrix2d<T> &);
/** Matrix subtraction m-n */
	friend rmMatrix2d<T> operator- <>(const rmMatrix2d<T> &,const rmMatrix2d<T> &);

private:
 	void allocateMatrix();
	void deleteMatrix();
};

/** overloaded ostream operator (cout)*/
	template<class T>
	ostream& operator<< (ostream& os, const rmMatrix2d<T>& m);

// done this for compilation using makefiles...
#include "rmmatrix2d.cpp"
#endif 
