/***************************************************************************
                          rmvector.cpp  -  description
                             -------------------
    begin                : Wed Jun 12 2002
    copyright            : (C) 2002 by Robert Marti
    email                : robert@localhost.localdomain
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* Define as we already include the implementation in the header (to avoid
redefinition*/
#ifndef RMVECTOR_C
#define RMVECTOR_C

#include "rmvector.h"
#include "caract/rmlib/rmutils/rmutils.h"
#include <libgen.h>
#include <stdio.h>

template<class T>
rmVector<T>::rmVector()
{
	data = NULL;
	dimX=0;
}

template<class T>
rmVector<T>::rmVector(int dx)
{
	data = NULL;
	dimX=0;
	resize(dx);
}

template<class T>
rmVector<T>::rmVector(const rmVector<T> &im1)
{
	int i;

	data = NULL;
	resize(im1.dimX);
	for(i=0;i<dimX;i++)
  	data[i] = im1(i);
}

template<class T>
const T & rmVector<T>::operator() (int x) const
{
	return data[x];
}

template<class T>
T & rmVector<T>::operator() (int x)
{
	return data[x];
}

template<class T>
void rmVector<T>::resize(int dx)
{
	deleteVector();
	dimX=dx;
	allocateVector();
}


template<class T>
rmVector<T>::~rmVector()
{
   deleteVector();
}

template<class T>
void rmVector<T>::deleteVector()
{
   delete[] data;
   dimX=0;
}

template<class T>
void rmVector<T>::allocateVector()
{
   data = new T [dimX];
}

template<class T>
rmVector<T> operator*(const rmVector<T> & v1,const rmVector<T> & v2)
{
	int i;
	rmVector<T> res;

	if (v1.dimX != v2.dimX) {
		error("Product: vectors should have same dimensions");
	}
	res.resize(v1.dimX);	
	for(i=0;i<v1.dimX;i++)
			res(i) = v1(i) * v2(i);
	return res;
}

template<class T>
rmVector<T> operator+(const rmVector<T> & v1,const rmVector<T> & v2)
{

	int i;
	rmVector<T> res;

	if (v1.dimX != v2.dimX) {
		error("Sum: vectors should have same dimensions");
	}
	res.resize(v1.dimX);	
	for(i=0;i<v1.dimX;i++)
			res(i) = v1(i) + v2(i);
	return res;
}

template<class T>
rmVector<T> operator-(const rmVector<T> & v1,const rmVector<T> & v2)
{
	int i;
	rmVector<T> res;

	if (v1.dimX != v2.dimX) {
		error("Subtraction: vectors should have same dimensions");
	}
	res.resize(v1.dimX);	
	for(i=0;i<v1.dimX;i++)
			res(i) = v1(i) - v2(i);
	return res;
}

template<class T>
rmVector<T> operator*(const rmVector<T> & v,double x)
{
	int i;
	rmVector<T> res;

	res.resize(v.dimX);	
	for(i=0;i<v.dimX;i++)
			res(i) = v(i) * x;
	return res;
}

template<class T>
rmVector<T> operator*(double x, const rmVector<T> & v)
{
	return T(v * x);
}


template<class T>
rmVector<T> operator+(const rmVector<T> & v,double x)
{
	int i;
	rmVector<T> res;

	res.resize(v.dimX);	
	for(i=0;i<v.dimX;i++)
			res(i) = T(v(i) + x);
	return res;
}

template<class T>
rmVector<T> operator+(double x, const rmVector<T> & v)
{
	return T(v + x);
}

template<class T>
rmVector<T> operator-(const rmVector<T> & v, double x)
{
	int i;
	rmVector<T> res;

	res.resize(v.dimX);	
	for(i=0;i<v.dimX;i++)
			res(i) = T(v(i) - x);
	return res;
}
template<class T>
rmVector<T> operator-(double x, const rmVector<T> & v)
{
	return T(v - x);
}

template<class T>
void  rmVector<T>::operator =(const rmVector<T> &im1)
{
	int i;

	resize(im1.dimX);
	for(i=0;i<dimX;i++)
  	data[i] = im1(i);
}

template<class T>
void rmVector<T>::printInfo()
{
	cout << "Vector dimension "<< dimX << endl;
}
#endif
