/***************************************************************************
                          regiongrowing.cpp  -  description
                             -------------------
    begin                : Tue Jul 9 2002
    copyright            : (C) 2002 by Robert Marti
    email                : robertmarti@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "rmregiongrowing.h"
#include "caract/rmlib/rmutils/rmutils.h"

//#include "Morphology/Morphology.h"

rmRegionGrowing::rmRegionGrowing()
{
	th = 50;
}

rmRegionGrowing::~rmRegionGrowing()
{
}

void rmRegionGrowing::runSeeded(rmImage3d &im, rmPoint3d p)
{
	int value;
	rmStatRegion sr;

	v3.resize(im.dimX,im.dimY,im.dimZ);
	im3 = im;
	
	stats.clear();
	nSeed=1;
	dimX= im.dimX;
	dimY= im.dimY;
	dimZ= im.dimZ;
	
	stack3d.clear();
	
	cout << "Seed point "<< p.x() << " "<< p.y()<< " "<< p.z() <<endl;
	value =im3(p.x(),p.y(),p.z());
	if (value==0){
		cerr<<"Error Seed is 0!"<<endl;
		abort();
	}

	sr.initStats(value,nSeed);
	v3(p.x(),p.y(),p.z())=nSeed;	//	pixel visited
	stack3d.push_back(p);
	checkIterative3d(sr);
	stats.push_back(sr);	// statistics of the new region.
	cout <<" Finished" << endl;
	v3.saveImage("seed");
	im= v3;
}

// 3d version
void rmRegionGrowing::doRunIterative(rmImage3d &im)
{
	int i,j,k,value;
	rmStatRegion sr;


	v3.resize(im.dimX,im.dimY,im.dimZ);
	im3 = im;
	
	stats.clear();
	nSeed=1;
	dimX= im.dimX;
	dimY= im.dimY;
	dimZ= im.dimZ;
	rmPoint3d p;
	
	stack3d.clear();
	
	cout << "initialized" << endl;
	
	for (i=0;i<im.dimX;i++)	
	  for (j=0;j<im.dimY;j++)	
	   	for (k=0;k<im.dimZ;k++) {
	   		if (v3(i,j,k)==0) // not visited
			{
			
				value =im3(i,j,k);

				sr.initStats(value,nSeed);
				v3(i,j,k)=nSeed;	//	pixel visited

				p.x(i);
				p.y(j);
				p.z(k);
				stack3d.push_back(p);
				
			//	cout << "->>New Region @ "<< i <<","<< j <<","<< k
			//		<< " sr "<< sr.N <<","<< sr.mean<< ","
			//	<< sr.var << " data " <<  im3.data[i][j][k]<< endl;
				
				checkIterative3d(sr);
				nSeed++;	//increase seeds(or regions);
				
				//cout << "->>Ending Region @ "<< i <<","<< j <<","<< k
				//<< " sr "<< sr.N <<","<< sr.mean<< ","
				//<< sr.var << endl;
				//getchar();
			
				stats.push_back(sr);	// statistics of the new region.
				
			}
	}
	
	cout <<" Finished, "<< nSeed-1 << " regions found. " << endl;
	//v3.saveImage("visits");
	extractMaximumVariance();
	//mask.saveImage("mask");
//	processMask();
	im= mask;
	cout << "finished"<< endl;
	
}

void rmRegionGrowing::checkIterative3d(rmStatRegion &sr)
{
	int i,j,k;
	rmPoint3d p,q;
	int value;
	
	
	while (!stack3d.empty())
	{
		p= stack3d.front();	// take first of the stack.
		stack3d.pop_front();
		
		for (i=p.x()-1;i<=p.x()+1;i++)
			for (j=p.y()-1;j<=p.y()+1;j++)
  			for (k=p.z()-1;k<=p.z()+1;k++){
			
	 			if ((i>=0) && (i<dimX) && (j>=0) && (j<dimY)
					&& (k>=0)&&(k<dimZ))
				{
					if ((v3(i,j,k)==0)
						&& condition2(im3(i,j,k),sr))
					{
					
						v3(i,j,k)=sr.getID();
						// belongs now to the region
						//updateStats((int)im3.data[i][j][k]);
						value = (int) im3(i,j,k);
						
						sr.updateStats(value);
												
						q.x(i);
						q.y(j);
						q.z(k);
						stack3d.push_back(q);
					}
					//getchar();
				}
		}
	}
}

int rmRegionGrowing::condition1(int value,rmStatRegion sr)
{
	return (abs(int(value-sr.getMean()))<th);

}

int rmRegionGrowing::condition2(int value, rmStatRegion sr)
{
	double cost=0;
	if (sr.getVar()!=0) {
		cost = (1/th)*((value-sr.getMean())*
			(value-sr.getMean())/sr.getVar());
	} else {
		cost = (1/th)*(value-sr.getMean())*(value-sr.getMean());
		 //fabs(value-sr.mean);
	}
//	cout << "cost "<< cost << endl;
	//getchar();
	return (cost<1);
}


/*void rmRegionGrowing::updateStats(int value)
{
	sr.sum+=value;
	sr.sum2+=(value*value);
	sr.N++;
	sr.mean = sr.sum/sr.N;
	sr.var = ((sr.sum2-sr.N*sr.mean*sr.mean)/(sr.N-1));
	//sr.stdev = sqrt(sr.var);
	// not doing stdev, saving computation.
}
*/

void rmRegionGrowing::setThreshold(double t)
{
	th=t;
}

double rmRegionGrowing::getThreshold()
{
	return th;
}


//////////////////////
// Extracts the region wich has maximum variance and puts it into an image
/////////////////////

//void rmRegionGrowing::extractMaximumVariance(rmImage3d &im)
void  rmRegionGrowing::extractMaximumVariance()
{

	int i,j,k;
	int pos,regID;
	double maxVar;
	list<rmStatRegion>::iterator it;


	//mask.resize(v3.dimX, v3.dimY, v3.dimZ);
	//mask.printInfo();
	//mask = im3;
	mask.resize(v3.dimX, v3.dimY, v3.dimZ);
	mask.printInfo();
	
	v3.printInfo();
	

	it = stats.begin();		
	pos = 0;
	regID = (*it).getID();
	maxVar= (*it).getVar();

	i=1;
	for (it=stats.begin()++; it!=stats.end(); it++)
	{	
		if (maxVar< (*it).getVar())
		{
			regID = (*it).getID();
			maxVar = (*it).getVar();
			pos =i;
		}
		i++;
	}
	cout << "Maximum variance " << maxVar << "found in region "<<
		regID << "ipos: " << pos<< endl;
	
	for (i=0;i<v3.dimX;i++)
	  for (j=0;j<v3.dimY;j++)
	  	for (k=0;k<v3.dimZ;k++) {
		  	if (v3(i,j,k)==regID) 	// max variance region found.
		  	{
					mask(i,j,k) = 255;
				}
	}
}

/*
// Do opening
void rmRegionGrowing::processMask()
{

	int k;
 	rmImage slice;
	char fn[100];
   	int i,j;
   	double u,v;
	int size =2;
 	RD_List<RD_ImagePoint> kernel;
   	RD_ImagePoint el;
   	kernel.clear();
	int ne=1;
	int nd=0;
   	

   	// Using octogonal kernel of 5x3 radius
   	for (i=-size;i<=size;i++)
      for(j=-size;j<=size;j++) {
    	    el.x(i);el.y(j);
	    kernel.push_back(el);
   	}
   	el.x(-3);el.y(-1);kernel.push_back(el);
   	el.x(-3);el.y(-0);kernel.push_back(el);
   	el.x(-3);el.y(1);kernel.push_back(el);

   	el.x(3);el.y(-1);kernel.push_back(el);
   	el.x(3);el.y(-0);kernel.push_back(el);
   	el.x(3);el.y(1);kernel.push_back(el);

   	el.y(-3);el.x(-1);kernel.push_back(el);
   	el.y(-3);el.x(-0);kernel.push_back(el);
   	el.y(-3);el.x(1);kernel.push_back(el);

   	el.y(3);el.x(-1);kernel.push_back(el);
	el.y(3);el.x(-0);kernel.push_back(el);
   	el.y(3);el.x(1);kernel.push_back(el);

   	cout << "Open"<<endl;
	for (k=0;k<dimZ;k++) {
		mask.getSlice(2,k,slice);
		Morphology<imdata> morph(slice.data,kernel);
   		for (i=0;i<ne;i++)
        	morph.Erode();
   		for (i=0;i<nd;i++)
        	morph.Dilate();
   		morph.ResIm(slice.data);
		//sprintf(fn,"2open%03i.byte",k);
		//slice.Save(fn);
		mask.setSlice(2,k,slice);
	}
}
  */

