/***************************************************************************
                          rmcroscor.cpp  -  description
                             -------------------
    begin                : Fri Jul 5 2002
    copyright            : (C) 2002 by Robert Marti
    email                : robertmarti@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "rmcroscor.h"
#include <math.h>

rmCross::rmCross()
{
}

rmCross::~rmCross()
{
}

// Calculate mean intensity of the image
double rmCross::calcMean(const rmImage3d &A)
{
	int i,j,k;
	float mean=0.0;

	for (i=0;i<A.dimX;i++)
	  for (j=0;j<A.dimY;j++)
		for (k=0;k<A.dimZ;k++)
		{	
			mean += A(i,j,k);		
		}
	mean = mean/float(i*j*k);
	return mean;
}


double rmCross::eval(const rmImage3d &A, const rmImage3d &B)
{
	double num,dA,dB,res,meanA,meanB;
	int i,j,k;
	
	meanA=calcMean(A);
	meanB=calcMean(B);

	num=0.0;
	dA=0.0;
	dB=0.0;
	int dx,dy,dz;
	if (A.dimX>B.dimX) dx = B.dimX;
	else dx =A.dimX;
	if (A.dimY>B.dimY) dy = B.dimY;
	else dy =A.dimY;
	if (A.dimZ>B.dimZ) dz = B.dimZ;
	else dz =A.dimZ;
	for (i=0;i<dx;i++)
		for (j=0;j<dy;j++)
			for (k=0;k<dz;k++) {
			num+= (A(i,j,k)-meanA) * (B(i,j,k)-meanB);
			dA+= (A(i,j,k)-meanA) * (A(i,j,k)-meanA);
			dB+= (B(i,j,k)-meanB) * (B(i,j,k)-meanB);
	}
	res= num / (sqrt(dA)*sqrt(dB));
	return res;
}

// Calculate mean intensity of the image
double rmCross::calcMean(const rmImage & A)
{
	int i,j, count;
	float mean=0.0;
	
	count =0;
	for (i=0;i<A.dimX;i++)
		for (j=0;j<A.dimY;j++) {
			if (A.data(i,j)>0) {
				mean += A.data(i,j);		
				count++;
			}
	}
	mean = mean/float(count);

	return mean;
}


double rmCross::eval(const rmImage &A, const rmImage &B)
{
	double num,dA,dB,res,meanA,meanB;
	int i,j;
	
	meanA=calcMean(A);
	meanB=calcMean(B);

	num=0.0;
	dA=0.0;
	dB=0.0;
	int dx,dy;
	if (A.dimX>B.dimX) dx = B.dimX;
	else dx =A.dimX;
	if (A.dimY>B.dimY) dy = B.dimY;
	else dy =A.dimY;
	for (i=0;i<dx;i++)
		for (j=0;j<dy;j++)
		{
			if ((A.data(i,j)>0) && (B.data(i,j))>0)
			{
				num+= (A.data(i,j)-meanA) * (B.data(i,j)-meanB);
				dA+= (A.data(i,j)-meanA) * (A.data(i,j)-meanA);
				dB+= (B.data(i,j)-meanB) * (B.data(i,j)-meanB);
			}
		}
	res= num / (sqrt(dA)*sqrt(dB));
	return res;
}


double rmCross::evalRMS(const rmImage &A, const rmImage &B)
{
	double num,res;
	int i,j;
	long count;

	num=0.0;
	count=0;
	int dx,dy;
	if (A.dimX>B.dimX) dx = B.dimX;
	else dx =A.dimX;
	if (A.dimY>B.dimY) dy = B.dimY;
	else dy =A.dimY;
	for (i=0;i<dx;i++)
		for (j=0;j<dy;j++)
		{
			if ((A.data(i,j)>0) && (B.data(i,j)>0))
			{
				num+= (A.data(i,j)-B.data(i,j)) * (A.data(i,j)-B.data(i,j));
				count++;
			}
		}
	res= sqrt(num)/count;
	return res;
}

